%%  Ormsby Band-pass filter

%% Parameters:
%%      data - record to be filtered. If data is a 2D matrix, each column 
%%            of it is filtered independently.
%%      dt - sampling interval in this record
%%      filter_band - pass band of the filter: [ low_cut, low_pass, high_pass, high_cut ]
%%		in units in which 'fi' is returned by 'spectrum()' (usually kHz)
%%		If value of 'low_cut' is given as negative, the filter is implemented as band-reject,
%%		with band = abs(filter_band)
%%	reject - boolean parameter setting 'band-reject' filter (cutting the band instead of passing)
%%		If this parameter is not set, reject = false (ordinary band pass) is assumed
%%	notch - if not empty, array of frequencies at which notch filter is applied
%%
%%  Returns:
%%      d - filtered record of the same size as 'data'

function d = bp_filter( data, dt, filter_band, notch )
     
  if nargin < 3      % no band; return unfiltered data
    d = data;
    return
  end 
  
  if isempty(filter_band)
    	reject = true;
  elseif filter_band(1) < 0.0
	reject = true;
  else
	reject = false;
  end

  if nargin < 4      % no notch filter
    notch = [];
  end 

%filter_band
%reject

  filter_band = sort(abs(filter_band));		% ignore signs and sequence of parameters

  % pad records with zeros at the beginning

  npad = round(size(data,1)*0.25);
  data = [ zeros(npad,size(data,2)); data ];

  % spectrum and frequency sampling interval
  
  [ y fi fN ] = spectrum(data,dt);         
    
  % put constant for the part of the filter outside of the band
    
  if reject
 	filt = ones(size(y,1),1);
  else
  	filt = zeros(size(y,1),1);
  end
 
  % apply band-pass filter

  if length(filter_band)>=4
  	f = round(filter_band(2:3)/fi);    % filter pass/reject range
    
  	if f(1) < 1
    		f(1) = 1;
  	end 
  
  	if f(2) > fN
    		f(2) = fN;
  	end 

  	i0 = max([ round(filter_band(1)/fi) 1]);       % low-cut frequency
 
  	if f(1) > i0
    		a = pi/(f(1)-i0);       % factor to scale the ramp range to pi
  	else
    		a = 1.0;
  	end 
  
  	ramp = i0:(f(1)-1);       % low-cut ramp
    
%fi
%fN
%f
%ramp1 = ramp

  	if reject
   		filt(ramp,1) = 0.5*(1+cos(a*(ramp-i0)));
  		filt(f(1):f(2)) = 0.0;             		% flat part
  	else
  		filt(ramp,1) = 0.5*(1-cos(a*(ramp-i0)));
  		filt(f(1):f(2)) = 1.0;             		% flat part
  	end

  	i0 = min([ round(filter_band(4)/fi) fN]); % high-cut frequency
    
  	if i0 > f(2)
    		a = pi/(i0 - f(2));       % factor to scale the ramp range to pi
  	else
    		a = 1.0;
  	end
  
  	ramp = (f(2)+1):i0;         % high-cut ramp
  
%ramp2 = ramp

  	if reject
		filt(ramp,1) = 0.5*(1.+cos(a*(ramp-i0)));
   	else
		filt(ramp,1) = 0.5*(1.-cos(a*(ramp-i0)));
  	end 

  end	% end if bandpass filter

  % superimpose notch filter

  for k=1:length(notch)
	
  	i0 = round(notch(k)/fi);    % filter frequency index

  	i1 = round(notch(k)*1.05/fi);	% top of 10% notch band
 
	if i1 >= fN		% notch above Nyquist, skip it
		continue
	end

  	a = pi/(i1-i0);       % factor to scale the ramp range to pi
  
  	ramp = i0:i1;         % upward ramp
    
	filt(ramp,1) = 0.5*(1-cos(a*(ramp-i0)));

  	i1 = round(notch(k)*0.95/fi);	% bottom of 10% notch band
 
  	a = pi/(i1-i0);       % factor to scale the ramp range to pi
  
  	ramp = i1:i0;         % downward ramp
    
	filt(ramp,1) = 0.5*(1-cos(a*(ramp-i0)));
   end

%filt

  n = size(filt,1);
  
  filt(fN:end,1) = filt((end-fN+1):-1:1,1);     % mirror the band above Nyquist
 
        % test plot
      
%  figure(777); 
%  plot([1:size(y,1)]'*fi,100*filt,'-k');
%  plot([1:size(y,1)]'*fi,abs(y),'-b');
 
    % multiply columns by filter 
 
  for i=1:size(y,2)
    y(:,i) = y(:,i).*filt;
  end % for loop

      % complete test plot
      
  %plot([1:size(y,1)]'*fi,abs(y),';filt. spectrum;');    
  
  d = real(ifft(y));   % inverse FFT, wipe out the small imaginary part
  
  d = d((npad+1):end,:);        % drop padding on return
  
end % function


