%% Calculates Fourier spectra of all records from data/ directory, 
%% saves them in structure 'spec' in the same .mat files,
%% and displays in Figure 21
%%
%% General processing parameters need to be provided in structure 'procdata':
%%	procdata.time_range = (for example) [ -10 0 35 40 ] 	
%%		Analysis time window:
%%		[ left end of taper, start of record, end of record, right end of taper ]
%% Parameters:
%%	ffid - FFID number for which to create spectra
%%
%% Usage examples:
%%	plot_spectra(150)

function create_spectra(ffid)

global procdata

  % load record

  [ rec, ~, ~, dataf ] = record_ffid(ffid);

  if isempty(rec)
	disp(sprintf("create_spectra: Record for FFID %d is empty", ffid))
	return
  end

  %% save parameters of time sampling in 'spectr'

  spec.dt = rec.times(2)-rec.times(1);	% nominal sampling interval 
					% for spectral calculations

  spec.time_range = time_range(rec.offset);

  %% determine smallest number of samples in all records

  ns = fix(min(spec.time_range(:,4)-spec.time_range(:,1)) / spec.dt);

  %% extract time segment for spectral analysis and taper its ends:

  d = zeros(ns+1,size(rec.data,2));

  for i=1:size(rec.data,2)
  	ind = find(rec.times > spec.time_range(i,1));	

	ind = ind(1):(ind(1)+ns);	% rows we need to keep

  	t = rec.times(ind,:);

  	d(:,i) = rec.data(ind,i).*taper_cos(t, spec.time_range(i,2),spec.time_range(i,1)) ...
				.* taper_cos(t, spec.time_range(i,3),spec.time_range(i,4));

	% correct the time ranges for exact times of samples

	spec.time_range(i,:) = spec.time_range(i,:) + (t(1) - spec.time_range(i,1));
  end

  %% 1) calculate complex spectra of field records (excluding the times)

  [ spec.data, spec.fi, spec.fN ] = spectrum(d, spec.dt);

  %% form column of output frequencies (from 0 to Nyquist)

  f = transpose(0:(spec.fN-1))*spec.fi;

  %% truncate the spectra to below Nyquist frequency

  spec.data((spec.fN+1):end,:) = [];

  %% apply to spectra phase shifts related to nonzero time starts 

  ef = exp(f*(-2i)*pi*transpose(spec.time_range(:,2)));	% matrix of phase shifts for spectra

  spec.data = spec.data .* ef;

  %% 2) plot logarithms of average amplitude spectra within three offset ranges

  aoffs = abs(rec.offset);

  maxdist = max(aoffs);		% maximum distance

  run('../config/settings.m')		% obtain settings in structure 'sett'

  figure(21);		% use figure numbers that do not overlap with waveform plots,
			% so that you can run various scripts simultaneously
  clf
  set(gcf,'Position',sett.window_wide);

  subplot(1,3,1)		% plot waveforms

  plot_ffid1(rec,0.6,spec.time_range)

  subplot(1,3,2:3)		% plot spectra
  hold on

  linestyle = [ '-b'; '-r'; '-g'; '-m'; '-k' ];

  leg = [];	% plot legend

	% plot by channel ranges

  for i = 1:size(procdata.chan_range,1)	
  	d1 = procdata.chan_range(i,:);

  	ind1 = find(rec.chan >= d1(1) & rec.chan <= d1(2) );

  	if ~isempty(ind1)
		plot(f,10*log10(mean(abs(spec.data(:,ind1).^2),2)),linestyle(rem(i,5)+1,:))

		if isempty(leg)
  			leg = sprintf("chan %d - %d", d1(1), d1(2) );
		else
  			leg = [ leg; sprintf("chan %d - %d", d1(1), d1(2) ) ];
		end
  	end
  end

	% plot by distance ranges

  for i = 1:size(procdata.offset_range,1)	
  	d1 = procdata.offset_range(i,:);

  	ind1 = find(aoffs >= d1(1) & aoffs < d1(2) );

  	if ~isempty(ind1)
		plot(f,10*log10(mean(abs(spec.data(:,ind1).^2),2)),linestyle(rem(i,5)+1,:))

		if isempty(leg)
  			leg = sprintf("%1.1f - %1.1f m", d1(1), d1(2) );
		else
  			leg = [ leg; sprintf("%1.1f - %1.1f m", d1(1), d1(2) ) ];
		end
  	end
  end

  legend(leg)

  xlabel('Frequency (kHz)')
  ylabel('Amplitude (dB)')
  grid on

  if procdata.print_vintage
  	title(sprintf('Amplitude spectra in Line %d, Vint %d, FFID %d',rec.line(1), rec.vintage, rec.ffid ))
  else
  	title(sprintf('Amplitude spectra in Line %d, FFID %d',rec.line(1), rec.ffid))
  end

  % output the figure in jpeg

  warning("off"); 		% suppress annoying warnings from print()

  dirname = sprintf('plots/spectra/line_%03d',rec.line(1));

  mkdir('plots/spectra/');
  mkdir(dirname);

  if procdata.print_vintage 
	fn = sprintf('spec_v%04d_f%05d%s',rec.vintage,rec.ffid,procdata.plot_suffix);
  else
	fn = sprintf('spec_f%05d%s',rec.ffid,procdata.plot_suffix);
  end

  figure_jpeg(dirname,fn);

  warning("on"); 

  % save data into the same file in binary Matlab format
  
  if inOctave()
  	save(dataf,'-mat7-binary','rec','spec');
  else
	save(dataf,'rec','spec');
  end
end

