%% Loads one raw shot record by FFID number from rdirectory 'mat/',
%% applies editing, and saves in directory 'data/'.
%%
%% Editing includes:
%%	setting 'kill_list' flags
%%	picking approximate time of the source
%%	shifting time values to equal 0 at the picked source
%%	reducing the time range  
%%
%% Editing parameters can be provided in global structure 'procdata'
%% and also given in the arguments of this function.
%%
%% The edited record is saved in directory 'data/'
%% ====================================================================
%%
%% Parameters:
%%	ffid - ffid number to plot and edit
%%	kill_list - list of channels to mark as killed, in addition to those given in 'procdata.kill_list'
%%	time_start - start (source) time value in the record. If this argument is empty ([]),
%%		time_start is auto-picked  by linear regression of the times of the largest positive 
%%		values within 10 channels near the source
%%	time_range - time window to extract into the output records, relative to the time_start.
%%		For example, time_range  = [-100, 1000] means that the output time values 
%%		will start at -100 ms and end at 1000 ms.
%%		If this argument is empty ([]), procdata.time_range is used.
%%		If procdata.time_range is empty, the entire time range of raw records is output.
%% Returns:
%%	rec - edited record structure
%%
%% Usage examples:
%%	rec = edit_ffid(150);

function rec = edit_ffid(ffid, kill_list, time_start, time_range)

global procdata

  if nargin < 2
	kill_list = [];
  end

  if nargin < 3
	time_start=[];
  end

  if nargin < 4
	time_range=[];
  end

  %% load record

  rec = record_ffid(ffid,'raw');

  if isempty(rec)
	disp(sprintf("FFID %d not found in log files",ffid))
	return
  end

  %% drop unused channels

  rec.data(:,procdata.drop_chan) = [];
  rec.chan(procdata.drop_chan) = [];
  rec.rec_stn(procdata.drop_chan) = [];
  rec.offset(procdata.drop_chan) = [];

  %% combine killed records with those listed in procdata

  rec.kill = unique([ rec.kill, kill_list, procdata.kill ]);

  %% cancel kill for channels listed in 'unkill' list

  rec.kill(find(ismember(rec.kill, procdata.unkill))) = [];
	
  %% completely drop killed records if requested

  if procdata.drop_killed 
  	ind = find(ismember(rec.chan,rec.kill));

	rec.data(:,ind) = [];
	rec.chan(:,ind) = [];
  	rec.rec_stn(:,ind) = [];
  	rec.offset(:,ind) = [];

	% other parameters will be recalculated for the remaining rec.chan below
  end

  %% If needed, use custom scheme for station numbering based on ffid and chan 

  if exist('survey_rec.m')	
  	rec.rec_stn = survey_rec(rec.ffid,rec.chan);
  end

  if exist('survey_src_old_Marks.m')		% custom function for old Mark's files

  	[rec.src_stn, rec.src_x, rec.src_skid, rec.src_offset ] = survey_src(rec.ffid);

  elseif exist('survey_src.m')	% maybe custom function for sources will be needed?

  end	

  %% renumber source stations if surveyed locations do not match receiver stations

  if exist('survey_stn.m')		
  	rec.src_stn = survey_stn(rec.src_stn);
  end

  %% correct the source position if needed

  if abs(procdata.src_chan_shift) > 1.0e-5	% shift source station number
  	j = floor(procdata.src_chan_shift);

  	rec.src_stn = rec.src_stn + j;

  	rec.src_skid = procdata.src_chan_shift - j*rec.line(4);
  end

  %% determine inline and 3-D coordinates of source and receivers
  %% -----------------------------------------------------------------------
  %% if pattern is given, read coordinates and offsets from it

  if rec.pattern > 0

	[ rec.src_xyz, rec.rec_xyz, rec.src_x, rec.rec_x ] = ...
		read_pattern( rec.pattern, rec.src_stn, rec.src_skid, rec.src_offset, rec.rec_stn );

  else		% without pattern, use station numbers and custom functions

	[ rec.src_xyz, rec.rec_xyz, rec.src_x, rec.rec_x ] = ...
			set_coords( rec.line, rec.src_stn, rec.src_skid, rec.src_offset, rec.rec_stn );
  end

  % change the integer part of source station number to the nearest receiver station

  if procdata.src_stn_correct		

	[ ~, j ] = min(abs( rec.rec_x - rec.src_x ));	% index of nearest receiver

	rec.src_stn = rec.rec_stn(j);
	rec.src_skid = rec.src_x - rec.rec_x(j);
  end

  %% calculate source-receiver offsets

  if procdata.offsets_xyz	% use 3-D offsets

	rec.offset = reshape(distance(rec.rec_xyz,rec.src_xyz),1,[]).*sign(rec.rec_x - rec.src_x);	
							% save as a row

  else			% use inline offset (maybe if geometry is inaccurate) 

  	rec.offset = sqrt((rec.rec_x - rec.src_x).^2 + rec.src_offset.^2).*sign(rec.rec_x - rec.src_x);
  end

  % test printouts

%rline = rec.line
%src = [ rec.src_stn, rec.src_skid, rec.src_offset, rec.src_x, rec.src_xyz ]	
%rx = [ rec.rec_stn', rec.rec_x', rec.rec_xyz, rec.offset' ]

  % filter records for plotting and picking

  data = bp_filter( rec.data, rec.dt, procdata.filter_bp, procdata.filter_notch );

  %% pick first arrival times and measure amplitudes

  rec.picks_aux = pick_fbreaks(rec,[]);		% picking aux record

  rec.picks_fb = pick_fbreaks(rec,data);

  % data = stalta(data,rec.dt);		% test by plotting STA/LTA ratio instead of data

  %% derive travel-time model for near traces

  if ~isempty(rec.picks_fb) && procdata.pick_fb.near_trc > 0

	% select traces for determining source time:

	if isempty(procdata.pick_fb.near_chan)
		fb = rec.picks_fb;
	else
		ind = find(rec.chan>=procdata.pick_fb.near_chan(1) & rec.chan<=procdata.pick_fb.near_chan(2));
						% columns in rec.data

		ind = find(ismember(rec.picks_fb(:,1),ind));	% indices in rec.picks_fb

		fb = rec.picks_fb(ind,:)
	end

	% to select up to 'procdata.pick_fb.near_trc' traces, first sort and then drop the tail

	switch lower(procdata.pick_fb.pick_source_mode)

	  case 'maxamp'				% with largest amplitudes

  		[ ~, ind1 ] = sort(fb(:,4),'descend');

	  case { 'minoffset', 'minoffs' }	% with smallest unsigned offsets

  		[ ~, ind1 ] = sort(abs(rec.offset(fb(:,1))));

	  case 'mintime'			% with earliest times

  		[ ~, ind1 ] = sort(fb(:,2));
	end

	n = min([length(ind1),procdata.pick_fb.near_trc]);

	fb = fb(ind1(1:n),:);   % near picks sorted in requested way
  
	[ ~, ind ] = sort(fb(:,1));		% make sure channels are in sequence (for plotting)
	fb = fb(ind,:);

	ind_near = fb(:,1);

  	o = abs(rec.offset(ind_near));		% unsigned offsets
  	t = transpose(fb(:,2));			% times of peaks

  	rec.tt_near = polyfit1(o,t);	% linear regression of near-source times

%fb
%o
%t
%rtt = rec.tt_near

  else
	rec.tt_near = [];
	ind_near = [];
  end

  %% estimate time of the source 

  switch lower(procdata.pick_source_time)

    case "aux"		% use picked onset in auxiliary record

	if ~isempty(rec.picks_aux)
		time_start = rec.picks_aux(1,2);
	end

    case "near"		% use linear regression of times in near traces

  	if isempty(time_start) && ~isempty(rec.tt_near)

  		time_start = polyval(rec.tt_near,0.0);	% time at zero offset
	end

    otherwise

	time_start = 0;
  end

  % if requested, shift time start for the whole ffid

  if ~isempty(procdata.time_shift) 
  	ind = find( procdata.time_shift(:,1) == rec.ffid );

  	if ~isempty(ind)
		time_start = time_start + procdata.time_shift(ind,2);
  	end
  end

%rfb = [ rec.picks_fb, rec.offset' ]
%time_start

%time_start = 0;  % test

  rec.time_start = time_start;

  %% subtract source time from time grid, picks, and polyfit model

  rec.times = rec.times - time_start;

  if ~isempty(rec.picks_aux)
  	rec.picks_aux(:,2) = rec.picks_aux(:,2) - time_start;
  end

  if ~isempty(rec.picks_fb)
	rec.picks_fb(:,2) = rec.picks_fb(:,2) - time_start;
  end

  if ~isempty(rec.tt_near)
	rec.tt_near(end) = rec.tt_near(end) - time_start;
  end

  %% truncate the time range

  if isempty(time_range)
	time_range = procdata.time_range;
  end

  if ~isempty(time_range)
	ind1 = find(rec.times(:,1) >= time_range(1) & rec.times(:,1) <= time_range(2));

	if ~isempty(rec.aux)
		rec.aux = rec.aux(ind1,:);
	end

	rec.times = rec.times(ind1,:);
	rec.data = rec.data(ind1,:);
	data = data(ind1,:);		% filtered version of rec.data
  end

  % save data in binary Matlab format
  
  if procdata.save

	% create directory for edited files in case it does not exist

	warning off
	mkdir('data/');
	warning on

  	dataf = sprintf('data/f%05d.mat',rec.ffid);

  	if inOctave()
  		save(dataf,'-mat7-binary','rec');
  	else
		save(dataf,'rec');
  	end
  end

  if ~procdata.plot
	disp(sprintf("edit_ffid: Processed ffid %d",rec.ffid))
	return
  end

  % measure peak amplitudes in aux and filtered data channels

  if ~isempty(rec.aux)
	ampl_aux = max(abs(rec.aux))+1.e-8;
  end

  ampl = max(abs(data))+1.e-8;

  % start plotting
  % =============================================================================

  if procdata.plot_panels == 3
  	panel_ampl = 4;
  	panel_offs = 5;			% this is actually the last one

  elseif  procdata.plot_panels == 2

  	panel_ampl = 4;
  	panel_offs = 4;
  else
  	panel_ampl = 3;
  	panel_offs = 3;		
  end

  gain = 0.6;	% plot excursion

  run('../config/settings.m')		% obtain settings in structure 'sett'

  figure(1)
  clf					% this clears the figure in case it already exists
  set(gcf,'Position',sett.window_wide);

  subplot(panel_offs,1,1:3)

  hold on	% this makes the different plot() commands to overlay the curves

  % draw a green circle at channel number corresponding to the position of the source and time=0

  if abs(rec.line(4) < 1.0e-8 )		% this is actually a disallowed mistake which should never happen
					% but it still causes annpying division by zero, 
					% and so let's protect from thjis division

  	plot(rec.src_stn - rec.rec_stn(1) + rec.chan(1),0.0,'go')
  else
  	plot(rec.src_stn+rec.src_skid/rec.line(4) - rec.rec_stn(1) + rec.chan(1),0.0,'go')
  end

  % draw velocity lines in green

  for i=1:length(procdata.plot_vel)

	t = (rec.offset-rec.offset(1))/procdata.plot_vel(i); 	% align at t=0 at chan=1

 	plot( rec.chan, t,'g-');
	text( rec.chan(end)+0.1, t(end),sprintf("V = %1.1f",procdata.plot_vel(i)))
  end

  % draw auxiliary record in magenta at chan = first_chan - 1

  auxchan = rec.chan(1)-1;

  if isempty(rec.aux)
  	chanrange = [auxchan, rec.chan(end)+1];
  else
 	plot(rec.aux/ampl_aux*gain+auxchan,rec.times,'m-')

  	chanrange = [auxchan-1, rec.chan(end)+1];
  end

  t = rec.times;

  for i=1:length(rec.chan)
	chan = rec.chan(i);

	d = data(:,i);

	% draw the trace (blue - good, red - killed)

	if ismember(chan,rec.kill)
 		plot(d/ampl(i)*gain+chan,t,'r-')
	else
 		plot(d/ampl(i)*gain+chan,t,'b-')
	end
  end

  % draw the tt model in near traces by black lines

  if ~isempty(rec.tt_near)
  	plot(rec.chan(ind_near),polyval(rec.tt_near,abs(rec.offset(ind_near))),'k-')
  end

  % draw crosses at first-arrival picks  

  if ~isempty(rec.picks_aux)
  	plot(auxchan, rec.picks_aux(:,2),'kx')
  end

  if ~isempty(rec.picks_fb)
  	plot(rec.chan(rec.picks_fb(:,1)),rec.picks_fb(:,2),'kx')
  end

  chanticks = rec.chan(1):procdata.plot_chan_increment:rec.chan(end);

  xlim(chanrange)

  if ~isempty(procdata.plot_time_range)
  	ylim(procdata.plot_time_range)
  elseif ~isempty(rec.times)
  	ylim([rec.times(1), rec.times(end)])
  end

  if procdata.plot_grid_on
  	grid on
  end

  set(gca,'Ydir','reverse')
  set(gca,'XTick',chanticks)

  ylabel("Time (ms)")
  xlabel(sprintf("Channel number and %s",procdata.rec_quantity))

  if procdata.print_vintage
  	str = sprintf("Line %d; Vint %d; FFID %d;", rec.line(1), rec.vintage, ffid );
  else
  	str = sprintf("Line %d; FFID %d;", rec.line(1), ffid );
  end

  title( sprintf("%s SRC at stn=%d skid=%1.2f %s offset=%1.2f %s; ts = %1.1f ms", ...
		str, rec.src_stn, rec.src_skid, procdata.unit_distance, ...
		rec.src_offset, procdata.unit_distance, time_start ) )

  if procdata.plot_panels < 2		% do not need amplitude plot

	figure_jpeg(sprintf('plots/records/line_%03d',rec.line(1)),...
			sprintf("rec_f%05d%s",rec.ffid,procdata.plot_suffix))
	return
  end

  subplot(panel_offs,1,panel_ampl)
  hold on

  if ~isempty(rec.aux)

  	plot(auxchan,20*log10(ampl_aux),'rx')
  end

  plot(rec.chan,20*log10(ampl),'bx')

  grid on
  xlim(chanrange)

  set(gca,'XTick',chanticks)

  ylabel("Amplitude (dB)")

  if procdata.plot_panels < 3		% do not need offset plot
	figure_jpeg(sprintf('plots/records/line_%03d',rec.line(1)),...
			sprintf("rec_f%05d%s",rec.ffid,procdata.plot_suffix))
	return
  end

  subplot(panel_offs,1,panel_offs)

%rffid = rec.ffid
%rchan = rec.chan
%roffs = rec.offset

  plot(rec.chan,rec.offset,'b-')

  grid on

  xlim(chanrange)

  set(gca,'XTick',chanticks)

  ylabel(sprintf("Offset (%s)",procdata.unit_distance))
  xlabel("Channel number")

  % create jpeg image

  if procdata.print_vintage 
	fn = sprintf("rec_v%04d_f%05d%s",rec.vintage,rec.ffid,procdata.plot_suffix);
  else
	fn = sprintf("rec_f%05d%s",rec.ffid,procdata.plot_suffix);
  end

  figure_jpeg(sprintf('plots/records/line_%03d',rec.line(1)),fn);

end

