%% Dispersion analysis for an FFID (slowness version)
%% =====================================================================
%% 	For a given FFID, this function places measured phase-velocity spectra 
%%	into structure 'dispsemb', displays them and saves in the same files as 
%%	data structures 'rec' and 'spec'
%%
%%	General processing parameters need to be provided in structure 'procdata'
%%
%%	Creates a plot in figure(31)
%%
%% Parameters:
%%	ffid - FFID number for which to create phase-velocity spectra

%% Usage examples:
%%	measure_dispersion(150)

function measure_dispersion(ffid)

global procdata

  % load record, spectra, and also obtain the file name

  [ rec, spec, ~, dataf ] = record_ffid(ffid);

  if isempty(rec)
	disp(sprintf("Record for FFID %d is empty", ffid))
	return
  elseif isempty(spec)
	disp(sprintf("No spectra for FFID %d", ffid))
	return
  end

  dispsemb = semblance_vphase(rec,spec,procdata);

  %% save dispersion measurement results
  
  if inOctave()
  	save(dataf,'-mat7-binary','rec','spec','dispsemb');
  else
  	save(dataf,'rec','spec','dispsemb');
  end

  aoffs = abs(rec.offset);

  %% plot the semblance and its peaks

  run('../config/settings.m')		% obtain settings in structure 'sett'

  figure(31);	
  clf
  set(gcf,'Position',sett.window_wide);	

  % plot waveforms

  subplot(1,3,1)	
  hold on

  plot_ffid1(rec,0.6,spec.time_range)

  % plot the moveout range by green lines

  plot( rec.chan, aoffs/dispsemb.phase_vel(1), '-g')
  plot( rec.chan, aoffs/dispsemb.phase_vel(2), '-g')

  t = aoffs/dispsemb.phase_vel_expected;

  plot( rec.chan, t, '--g')
  text( rec.chan(end) +0.5, t(end), ...
	sprintf("s=%1.1f v=%1.2f",1/dispsemb.phase_vel_expected,dispsemb.phase_vel_expected), ...
	'Color','g' )

  % plot the offset ranges by vertical magenta lines

  linestyle = '-m';

  for i=1:size(dispsemb.offset_range,1)

     for j=1:2
	ch = interp1(rec.offset,rec.chan,dispsemb.offset_range(i,j));

	t1 = interp1(rec.chan,spec.time_range(:,1),ch);
	t4 = interp1(rec.chan,spec.time_range(:,4),ch);

	plot( [ch ch], [ t1, t4 ],linestyle)

	ch = interp1(rec.offset,rec.chan,-dispsemb.offset_range(i,j));

	t1 = interp1(rec.chan,spec.time_range(:,1),ch);
	t4 = interp1(rec.chan,spec.time_range(:,4),ch);

	plot( [ch ch], [ t1, t4 ],linestyle)
     end

     linestyle = '--m';		% change line style for overtone modes
  end

  subplot(1,3,2:3)		% plot semblance

  plot_dispersion(dispsemb)

  if procdata.print_vintage 
  	title(sprintf('Phase semblance Line %d, Vint %d, FFID %d; chan %d - %d',...
			rec.line(1), rec.vintage, rec.ffid, dispsemb.chans(1), dispsemb.chans(end)))
  else
  	title(sprintf('Phase semblance Line %d, FFID %d; chan %d - %d',...
			rec.line(1), rec.ffid, dispsemb.chans(1), dispsemb.chans(end)))
  end

  % output the figure in jpeg

  warning("off"); 		% suppress annoying warnings from print()

  dirname = sprintf('plots/dispersion/line_%03d',rec.line(1));

  mkdir('plots/dispersion');
  mkdir(dirname);

  if procdata.print_vintage 
	fn = sprintf('disp_v%04d_f%05d%s',rec.vintage,rec.ffid,procdata.plot_suffix);
  else
	fn = sprintf('disp_f%05d%s',rec.ffid,procdata.plot_suffix);
  end

  figure_jpeg(dirname,fn);

  warning("on"); 

end

