%% Reads one FFID in and adds its contribution to dispersion spectra 
%% at each CMP using Hann-function weights
%%
%% The program expects that measurement is done on a certain line and vintage
%% specified in fields procdata.line(1) and procdata.vintage
%% Values of this field from the FIRST record in the call to this function
%% are used in file names of output .mat file and .jpeg figure files
%%
%% Parameters:
%%	ffid - FFID number for which to create phase-velocity spectra

%% Usage examples:
%%	measure_dispersion_cmp()

function measure_dispersion_cmp(ffid)

global procdata dispsemb

  % load record and spectra for current ffid

  [ rec, spec ] = record_ffid(ffid);

  if isempty(rec)	% no such record

	if procdata.line(1)
		msg = sprintf('measure_dispersion_cmp: no FFID=%d in line %d vintage %d',...
						ffid,procdata.line(1),procdata.vintage);
	else
		msg = sprintf('measure_dispersion_cmp: no FFID=%d',ffid );
	end

	disp(msg)

	return
  end

  % obtain selected frequency range, records, and spectra from current ffid

  [ spacing, freq_range, chind, f, slwns, spec1 ] = semblance_params(rec, spec);

  % form weight matrices

  [ weight, weight_cmp ] = semblance_weight(rec.src_x, rec.rec_x(chind), f );

  %% normalize the spectral amplitudes (keep only the phase factor)

  spec1 = spec1 ./ (abs(spec1)+1.0e-16);

  % evaluate (f,slowness) spectrum along the cmp_x curve
 
  Phi = semblance_vphase1( f,rec.offset(chind), spec1, slwns, weight);

  if procdata.run == 1		% first call; initialize output structure and arrays

  	% store in dispsemb key information from record

  	dispsemb.line = rec.line;
  	dispsemb.ffid = rec.ffid;
  	dispsemb.line = rec.line;
  	dispsemb.vintage = rec.vintage;

	dispsemb.freq_range = freq_range;
	dispsemb.f = f;
	dispsemb.slwns = slwns;
	dispsemb.spacing = spacing;

  	% copy processing parameters into dispsemb structure (for reference in the future)

  	dispsemb.phase_vel_expected = procdata.phase_vel_expected;
  	dispsemb.phase_vel = procdata.phase_vel;
  	dispsemb.offset_range = procdata.offset_range;
  	dispsemb.mode_separ = procdata.mode_separ;
  	dispsemb.freq_min = procdata.freq_min;

  	dispsemb.semblance = zeros(length(f),length(slwns),length(procdata.cmp_loc));

	for i=1:size(dispsemb.semblance,3)
		dispsemb.semblance(:,:,i) = weight_cmp(:,i).*Phi;
	end

	dispsemb.weight = weight_cmp;

  else				% accumulate the total semblance and weight

	for i=1:size(dispsemb.semblance,3)
		dispsemb.semblance(:,:,i) = dispsemb.semblance(:,:,i) + weight_cmp(:,i).*Phi;
	end

	dispsemb.weight = dispsemb.weight + weight_cmp;
  end	 

   % if this is the last ffid, finalize the processing and make plots

  if procdata.run == procdata.num_runs

	% scale result by the inverse sum of weights at every point

%sds = size(dispsemb.semblance)
%sdw = size(dispsemb.weight)

	for j=1:size(dispsemb.semblance,1)
		for i=1:size(dispsemb.semblance,3)
			dispsemb.semblance(j,:,i) = dispsemb.semblance(j,:,i)/(dispsemb.weight(j,i)+1.0e-16);
		end
	end

	disp(sprintf('measure_dispersion_cmp: Line %d, vintage %d: Finished %d ffids',...
					dispsemb.line(1), dispsemb.vintage, procdata.num_runs))

  	% save dispersion measurement results
  
	warning off
	mkdir('data/cmp/');
	warning on

	dataf = filename_dispersion(rec.line(1),rec.vintage);

  	if inOctave()
  		save(dataf,'-mat7-binary','dispsemb');
  	else
  		save(dataf,'dispsemb');
  	end

  	run('../config/settings.m')		% obtain settings in structure 'sett'

  	warning("off"); 		% suppress annoying warnings from print()

  	dirname = 'plots/dispersion_cmp/';

  	mkdir(dirname);

  	dirname = [ dirname, sprintf('line%03d_v%04d/', dispsemb.line(1), dispsemb.vintage) ];

  	mkdir(dirname);

  	%% plot the semblance and its peaks at each CMP

	for i=1:size(dispsemb.semblance,3)

		% pick peaks of semblance and mode separators
 
  		[ dispsemb.p_alias, dispsemb.p_separ, dispsemb.semb_peak, dispsemb.semb_peak1 ] = ...
					semblance_picks(dispsemb,i);
			
  		figure(41)	
  		clf
  		set(gcf,'Position',sett.window_narrow);

  		plot_dispersion(dispsemb,i)

  		title(sprintf('Phase semblance at CMP %d, x = %1.1f',i, procdata.cmp_loc(i)))

  		% output the figure in jpeg

  		print(sprintf('%s/disp_cmp%04d_l%04d_v%03d%s.jpg',dirname, ...
				i, dispsemb.line(1), dispsemb.vintage, procdata.plot_suffix),'-djpeg');
	end

  	%% plot (and jpg in the same directory) cross-sections at fixed slownesses

	for i=1:length(procdata.plot_disp_v)
	
		v = procdata.plot_disp_v(i);
		
  		figure(51)	
  		clf
  		set(gcf,'Position',sett.window_narrow);

  		plot_dispersion_v(dispsemb,v)

  		title(sprintf('Phase semblance at V = %1.3f m/ms',v))

  		% output the figure in jpeg

  		print(sprintf('%s/disp_v%1.3f_l%04d_v%03d%s.jpg',dirname, v, ...
					dispsemb.line(1), dispsemb.vintage, procdata.plot_suffix),'-djpeg');
	end

  	warning("on"); 

  end		% if last run

end

