% Obtains parameters of all FFIDs from the observer's log.
% =============================================================================
% The log is scanned for non-commentary entries and columns of
% line number values and FFID parameters output as in params_ffid.m
%
% Parameters:
%	log_file - log file name to use. If empty value [] is given, the standard name
%		obtained by file_names(ffid) is used
%	line_required - if this parameter is given and > 0, only records for the given
%		line number are returned
%	vintage_required - if this parameter is given, not empty, and >= 0, only records 
%		for the given vintage number are returned
% Returns:
%	line_par - line parameters: matrix with 12 numbers per row. 
%			See header of params_header.m for a list of these values
%	ffid - column of FFID numbers (the same length as line_par)
%	src_no - column of source point numbers
%	src_stn - column of source station numbers
%	rec_stn - matrix of rows containing receiver station numbers for the first and last receiver
%	trigger - column of trigger numbers
%	vintage - column of vintage numbers (-1 if vintage is not specified)

function [ line_par, ffid, src_no, src_stn, rec_stn, trigger, vintage ] = params_line(log_file, line_required, vintage_required)

	if isempty(log_file)
		log_file = file_names(ffid);
	end

	fid = fopen(log_file,'r');

	if fid < 0
		error("Cannot open file %s for reading",log_file)
	end

	line_par = [];
	ffid = [];
	src_no = [];
	src_stn = [];
	rec_stn = [];
	trigger = [];
	vintage = [];

	vint = -1;		% default vintage

	line_no = 0;		% default in case no line number in the header

	s = fgets(fid);		% read first line

	while s > 0

		s = strtrim(s);		% trim spaces

		if length(s) < 1		% skip blank line
			s = fgets(fid);			
			continue
		end

		if s(1) == '%' || s(1) == '#'		% skip commentaries 
							% but extract line or vintage numbers

			[ ~, s ] = strtok(s);		

			[ ind, tok ] = params_header(s);	% try recognizing a keyword

			n = str2num(tok);

			if ~isempty(n)		
				if ind == 1		% line number
					line_no = n;
				elseif ind == 12	% vintage number
					vint = n;
				end
			end

			s = fgets(fid);		
			continue
		end

		str = s;				% current line

		[ tok, s ] = strtok(s);			% first token

		line_par = [ line_par; line_no ];	% output line number

		src_no = [ src_no; str2num(tok) ];

		[ tok, s ] = strtok(s);			% second token

		ffid = [ ffid; str2num(tok) ];		% output FFID number

		% obtain other parameters

		[ tok, s ] = strtok(s);	

		src_stn = [ src_stn; str2num(tok) ];

		[ tok, s ] = strtok(s);	

		r(1) = str2num(tok);

		[ tok, s ] = strtok(s);	

		r(2) = str2num(tok);

		rec_stn = [ rec_stn; r ];

		[ tok, s ] = strtok(s);

		trigger = [ trigger; str2num(tok) ];

		vintage = [ vintage; vint ];

		s = fgets(fid);		% next line
	end
	
	fclose(fid);

	% select required line number

	if nargin >= 2 && line_required >= 1
		ind = find(line_par==line_required);
	else
		ind = transpose(1:length(line_par));
	end

	% select required vintage (and discard the selection by line number)

	if nargin >= 3 && vintage_required >= 0	

		ind = ind(find(vintage(ind,1)==vintage_required));
	end

	line_par = line_par(ind,:);
	ffid = ffid(ind,:);
	src_no = src_no(ind,:);
	src_stn = src_stn(ind,:);
	rec_stn = rec_stn(ind,:);
	trigger = trigger(ind,:);
	vintage = vintage(ind,:);
end

