%% Interactive picking of phase-semblance segments in an FFID or CMP
%%
%% The program expects that measurement is done on a certain line and vintage
%% specified in fields procdata.line(1) and procdata.vintage
%% Values of this field from the FIRST record in the call to this function
%% are used in file names of output .mat file and .jpeg figure files
%%
%% ===================================================================
%%
%%	The picking is conducted in figure(61)
%%
%% Parameters:
%%      num - requested FFID or CMP number 
%% Usage examples:
%%	pick_dispersion(150)

function pick = pick_dispersion(num)

global procdata

  % if full stop was requested, return without doing anything

  if strcmpi(procdata.run_status,'stop')
	return
  end

  dataset = lower(procdata.data_order);

  % file name for inputs and outputs

  fn = filename_interp(num,dataset);

  % load dispersion spectra from .mat files

  switch dataset
	case 'ffid'
  		[ rec, spec, dispsemb ] = record_ffid(num);

  		if isempty(rec)
			error('Record for num %d is empty', num)
  		elseif isempty(spec)
			error('Spectra for num %d have not been created', num)
 		elseif isempty(dispsemb)
			error('Dispersion semblance for num %d has not been created', num)
  		end

	case 'cmp'

		dataf = filename_dispersion(procdata.line(1),procdata.vintage);

		if exist(dataf)
			load(dataf)	% this should give structure 'dispsemb'
		else
			disp(sprintf('File %s does not exist',dataf))
			return
		end
  end

  % load or create parameters

  if exist(fn)
	load(fn)

	% temporary fix: replace with character arrays some strings that I had initially

	param.segment_list = {'N (highest)','N-1','N-2','N-3'};

  	param.title = sprintf('FFID %d, Line %d Vint %d', rec.ffid, rec.line(1), rec.vintage);

  else
	% if available, load picks from previous ones in the same session

%prun = procdata.run
%ppick = procdata.pick

	if procdata.run > 1 && ~isempty(procdata.pick)
		pick = procdata.pick;

	else				% create empty picks

  		pick.modes_xy = [];	% matrix containing all picks in two columns (frequency, slowness)

  		pick.modes = [ 	0 0	% indices of starts and ends of records of the modes in the above matrix
				0 0	% rows of this matrix correspond to the four modes being picked
				0 0
				0 0
		   		];		
	end

  	% store in pick key information from record

  	pick.number = dispsemb.ffid;
  	pick.id = rec.src_stn;
  	pick.line = dispsemb.line;
  	pick.vintage = dispsemb.vintage;

  	param = param_pick_dispersion();

	switch dataset
		case 'ffid'
  			param.title = sprintf('FFID %d, Line %d Vint %d', rec.ffid, rec.line(1), rec.vintage);

  			param.query_next = sprintf('STOP picking FFID=%d and go to the next FFID?',num);

			pick.id = rec.src_stn;

		case 'cmp'

  			param.title = sprintf('CMP %d, Line %d Vint %d', num, procdata.line(1),procdata.vintage);

  			param.query_next = sprintf('STOP picking CMP=%d and go to the next CMP?',num);

			pick.id = num;
	end

  end

  run('../config/settings.m')		% obtain settings in structure 'sett'

  figure(61);
  clf
  set(gcf,'Position',sett.window_narrow);

  switch dataset
	case 'ffid'

  		plot_dispersion(dispsemb)

	case 'cmp'

		% pick peaks of semblance and mode separators
 
  		[ dispsemb.p_alias, dispsemb.p_separ, dispsemb.semb_peak, dispsemb.semb_peak1 ] = ...
							semblance_picks(dispsemb,num);

  		plot_dispersion(dispsemb,num)
  end

  %% execute an interactive session modifying the pick segment data

  [ pick.modes_xy, pick.modes, procdata.run_status ] = pick_segments( pick.modes_xy, pick.modes, param);

  close(gcf)		% close figure window

  %% save updated dispersion records in interpretation file
  
  warning off
  mkdir('interp/');		% create directory in case it is absent
  warning on

  if inOctave()
%  	save(fn,'-mat7-binary','pick','param');
  else
  	save(fn,'pick','param');
  end

  %% keep in 'procdata' picks made in this session 

  procdata.pick = pick;

end

