%% Picks first arrivals in traces
%%
%% Picking is performed using the algorithm and parameters specified in
%% structure 'procdata.pick_fb':
%%
%%	if procdata.pick_fb.method == 'maxamp' (case-insentitive match), 
%%		simply picking the largest abs(amplitude)
%%	if procdata.pick_fb.method == 'stalta', picking using the STA/LTA method
%%
%% =============================================================================
%%
%% Parameters:
%%	rec - complete data record
%%	data - if this matrix is given, it contains traces used instead of 'rec.data'
%%		It can contain, for example, a filtered version of the data
%%		If this record is given but equal [], picking is done in rec.aux
%% Returns:
%%	pick_trc - matrix with four columns:
%%		1) numbers of columns in 'data' for which the outputs are made
%%		2) picked times for the traces in 'pick_col'
%%		3) row indices for the picks in 'pick_time' (i.e., times of the nearest sample)
%%		4) amplitudes for the picks in 'pick_time'
%%
%% Usage examples:
%%	rec = edit_ffid(150);

function pick_trc = pick_fbreaks(rec,data)

global procdata

  % do not consider killed records in picking

  pick_trc0 = find(~ismember(rec.chan,rec.kill));

  if nargin >= 2
	if isempty(data)
		if isempty(rec.aux)
			pick_trc = [];
			return
		else
			data = rec.aux;
		end
	else
		data = data(:,pick_trc0);
	end
  else
  	data = rec.data(:,pick_trc0);
  end

  [ nt, nd ] = size(data);	% number of times and traces

  int_st = round(procdata.pick_fb.t_peak/rec.dt);	% number of samples in 'peak' (STA) window

  % default values meaning no picks

  pick_trc = [];

  if strcmpi(procdata.pick_fb.method,'maxamp')

	pick_trc = transpose(1:nd);	% this is pick_trc(:,1) - picks made in all traces

  	[ pick_trc(:,3), ind_time ] = max(abs(data));	% largest abs(amplitude)

	pick_trc(:,2) = rec.times(ind_time);
	pick_trc(:,3) = ind_time;

  elseif strcmpi(procdata.pick_fb.method,'stalta')

  	int_lt = round(procdata.pick_fb.t_prepeak/rec.dt);	% number of samples in LTA window

	sta_lta = stalta(data,rec.dt);

	for i=1:nd
  		peak_stalta = max(sta_lta(:,i));	% largest STA/LTA

		if peak_stalta > procdata.pick_fb.stalta_min

			ind_time = find(sta_lta(:,i) > procdata.pick_fb.stalta_threshold*peak_stalta);
			
			if ~isempty(ind_time)
				pick_trc = [ pick_trc; i, rec.times(ind_time(1)), ind_time(1), 0.0 ];
			end
		end
	end

  else			% unknown method - no picks
	return
  end

  % adjust the picks if needed and measure RMS amplitudes within the 'peak' window
 
  if ~isempty(pick_trc)


	% measure RMS amplitudes

  	for i=1:size(pick_trc,1)
		i1 = pick_trc(i,3);
		i2 = i1 + int_st;

		i2 = min(nt,i2);	% truncate to the end of the trace 

		pick_trc(i,4) = sqrt(mean(data(i1:i2,pick_trc(i,1)).^2));
  	end

  	% return list of traces in the original sequence

  	pick_trc(:,1) = pick_trc0(pick_trc(:,1));
  end

end

