%% Plot picked wave modes for several lines and vintages.
%% Each line is plotted in a figure with number equal the number of the line,
%% and vintages are plotted using different line styles
%%
%% Plotting is performed by using proc_ffids()
%%
%% Parameters:
%%      line_list - list of line numbers to plot
%%	vint_list - list of vintages to plot
%%	vint_color - character matrix of color symbols for the vintages in 'vint_list'

function plot_modes_line(line_list, vint_list, vint_color )

global procdata modes

	if nargin < 2
  		vint_list = -1;		% all vintages
	end

	if nargin < 3
  		vint_color = [ 'k','r','g','b','c','m' ];
	end

	pdvl = procdata.vint_list;		% save the vintage list

  	run('../config/settings.m')		% obtain settings in structure 'sett'

	mode_name = [ 'N  '; 'N-1'; 'N-2'; 'N-3' ];	% mode names for legend

	% create lists of vintages, marker colors, and labels

	procdata.marker_color = vint_color;
	procdata.vint_list = vint_list;
  	procdata.plot_group = [];		% group not specified; will use vintage to form plot legend

	% create a double loop to process all lines and vintages desired
	% for each line, create a separate figure with plots of all vintages overlaid

	for line_no=line_list

  		procdata.lh = [];	% create persistent array of line handles

		figure(line_no)		% open a figure for each line
		clf
		set(gcf,'Position',sett.window_wide)

		procdata.line = line_no;

		for vint_no = vint_list

			procdata.vintage = vint_no;

			if isempty(procdata.subplots)		% make one pass through FFIDs or CMPs
				procdata.subplot = [];

				switch lower(procdata.dataset)
				  case 'id'
					proc_ids(@plot_modes)
				  otherwise
					proc_ffids(@plot_modes)
				end

  				grid on

				title(sprintf('Line %d: Dispersion picks from %s',line_no, procdata.dataset))

			else					% repeat passes for each subplot
				for i=1:size(procdata.subplots)
					procdata.subplot = procdata.subplots(i,:);

					switch lower(procdata.dataset)
				  	  case 'id'
						proc_ids(@plot_modes)
				  	  otherwise
						proc_ffids(@plot_modes)
					end

					if procdata.subplot(1) > 0	% valid plot, not legend only

					  if procdata.subplot(1) == procdata.subplot(2)				% only one ID
						title(sprintf('Line %d, : Dispersion picks from %s, ID %d',...
							line_no, procdata.dataset, procdata.subplot(1) ))
					  else									% ID range
						title(sprintf('Line %d, : Dispersion picks from %s, ID %d-%d',...
							line_no, procdata.dataset, procdata.subplot(1), procdata.subplot(2) ))
					  end
					end
				end
			end
		end

%plh = procdata.lh

  		procdata.lh = unique2(procdata.lh); 	% leave in 'lh' only one record per segment and vintage group

  		% display legend in the last panel 

  		if ~isempty(procdata.lh)
			lgd = mode_name(procdata.lh(:,1),:);

			lgd = [ lgd, repmat(' v',size(lgd,1),1), num2str(procdata.lh(:,2),'%-4d') ];

			if ~isempty(procdata.subplots)
				subplot(procdata.subplots(end,3),procdata.subplots(end,4),procdata.subplots(end,5))
				hold on
				%axes('Visible','off')
			end

			legend(procdata.lh(:,3),lgd,'location','southeast')
  		end

  		% output the figure in jpeg

  		warning("off"); 		% suppress annoying warnings from print()

		dirname = sprintf('plots/interp/line_%03d',line_no);

  		mkdir('plots/interp');
  		mkdir(dirname);
	
  		switch lower(procdata.dataset)
	  		case 'id'
				fn = sprintf('modes_l%04d_%s',line_no,lower(procdata.plot_modes));
 	 		otherwise
				fn = sprintf('modes_all_l%04d_%s',line_no,lower(procdata.plot_modes));
  		end

  		figure_jpeg(dirname,fn);

  		warning("on"); 

	end		% loop line

	procdata.vint_list = pdvl;		% restore the vintage list

end

