%%  Processes all FFID data files one by one using the specified program 
%%
%% Parameters:
%%	proc_fun - function handle or name for processing each ffid. This function
%%		should take a single argument equal the FFID number
%%	close_fun - function handle or name for finalizing processing after proc_fun()
%%		returns. If close_fun=[], this function is not called.
%%		For example, use close_fun = @ginput to pause processing 
%%		until the user presses a key.
%%		The function can also output strings that can be used to control processing.
%%		If close_fun() returns string 'stop' or 'q', processing is terminated.
%%	dataset - string parameter. If dataset='raw', raw field records
%%		from directory 'mat/' are used. If dataset='edited', [], or any other,
%%		edited data from directory 'data/' are used.
%%		If this parameter is not given, 'edited' is assumed
%%	select - list of ffids to select. If select==[], all ffids are tried
%%
%% Usage examples:
%%	proc_ffids(@edit_ffid,@ginput)
%%	proc_ffids(@edit_ffid,@ginput,'raw', [ 111, 120:130 ])

function proc_ffids(proc_fun, close_fun, dataset, select)

global procdata

  % if full stop was requested, return without doing anything

  if strcmpi(procdata.run_status,'stop')
	return
  end

  if nargin < 2
	close_fun=[];
  end

  if nargin < 3
	dataset=[];
  end

  if nargin < 4
	select=[];
  end

  %% list all ffids and process them one by one

  if isempty(dataset)
  	ffid = list_ffids('data/');
  elseif strcmpi(dataset,'raw')

  	if isdir("accel/")	% accelerometer data directory exists

		ffid = list_ffids('accel/');
	else
  		ffid = list_ffids('mat/');
	end
  else
  	ffid = list_ffids('data/');
  end

%dataset
%ffid

  if length(ffid) < 1
	disp(sprintf('proc_ffids: No FFIDs for line %d, vintage %d', procdata.line(1),procdata.vintage))
	return
  end

  if isempty(procdata.vintage)
  	disp(sprintf('proc_ffids: %d FFIDs for line %d, vintage []', length(ffid), procdata.line(1)))
  else
  	disp(sprintf('proc_ffids: %d FFIDs for line %d, vintage %d', length(ffid), procdata.line(1),procdata.vintage))
  end

  procdata.num_runs = length(ffid);	% total number of runs which can be used
					% in the functions called in the following loop

  procdata.run_status = 'run';		% use this field to pass instructions from called
					% function

  for i=1:procdata.num_runs

	if ismember(ffid(i),procdata.drop_ffid)		% skip FFIDs in drop list
		continue
	end

	if ~isempty(select)

		if ~ismember(ffid(i),select)		% skip
			continue
		end
	end

 	% disp(sprintf('proc_ffids: run %d, FFID %d',i,ffid(i)))

	procdata.run = i;		% run number in the call to proc_ffids
	procdata.run_ffid = ffid(i);	% ffid number in the call to proc_ffids

	proc_fun(ffid(i));

	if ~isempty(close_fun)
		procdata.run_status = close_fun();
	end

	if strcmpi(procdata.run_status,'stop')
		break
	end
  end

end

