%% Extracts one complete RGIM seismograph or accelerometer record by FFID number
%% ================================================================================

%% The record is extracted in the form of structure 'rec' below,
%% containing the following fields:
%%	rec.ffid - FFID number in this record
%%	rec.source - source station number from data file 
%%		(set during acquisition or edited later)
%%	rec.src_stn - source station number from data file 
%%	rec.chan - array (row) of channel numbers
%%	rec.rec_stn - receiver station numbers corresponding to each chan 
%%		In the initial data, these station numbers equal 'chan' values
%%	rec.offset - array of source-receiver offsets (distances) 
%%		In raw data, this equals (rec_stn - src_stn)
%%	rec.times - column of time sampling
%%	rec.data - matrix with rows corresponding to times and 
%%		 columns corresponding to recorded channels
%%	rec.kill - array of kill flags - numbers of channels to 
%%		skip in most processing. In the original record from the
%%		seismograph, this array is empty
%%	rec.trigger - trigger count number
%%	rec.trigger_time - array of trigger times from each of the 12 boards
%% ================================================================================
%% Parameters:
%%	ffid0 - requested ffid number
%%	dataset - optional string parameter. If dataset='raw', raw field records
%%		from directory 'mat/' are loaded. If dataset='edited' or any other,
%%		edited data from directory 'data/' are loaded.
%%		If this parameter is not given, 'edited' is assumed.
%% Returns:
%%	rec - structure containing the record (empty if no such record exists)
%%	spec - structure containing the record spectra (returned from loading
%%		edited and processed data)
%%	dispsemb - structure containing dispersion results
%%	datafile - name of .mat or .txt file from which the data were loaded
%%
%% Usage examples:
%%	rec1 = record_ffid(150)
%%	rec2 = record_ffid(150,'raw')

function [ rec, spec, dispsemb, datafile ] = record_ffid(ffid0,dataset)

global procdata

  if nargin < 1
	error("FFID not given")
  end

  if ismember(ffid0,procdata.drop_ffid)		% skip unwanted ffids
	rec=[];
	return
  end

  if nargin < 2
	dataset = 'edited';
  end

  [ logfile, datafile, filetype ] = file_names(ffid0,dataset);

%pline=procdata.line
%pvint=procdata.vintage
%ffid0
%dataset
%datafile
%filetype

  if exist(datafile)

	switch filetype

	  case 'seism'		% Octave field .mat file from seismograph

		load(datafile)

		rec.ffid = ffid;

		rec.chan = chan;

		if exist('offset')
			rec.offset = offset;
		else
			rec.offset = chan;
		end

		rec.data = data;
		rec.times = transpose(0:(size(data,1)-1))*timing(2) + timing(1);

  		rec.dt = timing(2);		% sampling interval

		rec.trigger = trigger;
		rec.trigger_time = trigger_time;

		rec.aux = [];	% no auxiliary records in this seismograph

		spec = [];	% no spectra for raw data
		dispsemb = [];

		% mark as killed records from failed data servers

		ind = find(AD_board==0);

		rec.kill = [];

		for i=1:length(ind)
			rec.kill = [ rec.kill, (1:8)+(ind(i)-1)*8 ];
		end

	  case 'accel'		% accelerometer 'Trial' file

		rec = record_accel(datafile);

		rec.ffid = ffid0;

		spec = [];	% spectra and dispersion are unavailable at this point
		dispsemb = [];

	  otherwise		% processed .mat file

		% check whether we are in Octave or Matlab

		if inOctave()
			load(datafile,'-mat')
		else
			load(datafile)		% this loads the entire structure 'rec'
		end

		if ~exist('spec')
			spec=[];
		end

		if ~exist('dispsemb')
			dispsemb=[];
		end

		return
	end

	% load shot params from log file 
	% (ignore 'trigger' because it should certainly be accurate in the .mat file)

	[ rec.line, rec.source, rec.src_stn, rstn, rec.pattern ] = params_ffid(logfile,ffid0);

	rec.rec_stn = rstn(1):rstn(2);

  	rec.src_skid = rec.line(7);		
  	rec.src_offset = rec.line(6);
	rec.vintage = rec.line(12);

  else
	rec = [];
	spec = [];
	dispsemb = [];
  end

%rkill = rec.kill

end

