%% Select the frequency and offset range for dispersion semblance calculation
%% ===================================================================================
%%
%% Parameters:
%%	rec - time-domain record
%%	spec - spectra for the record
%%
%% Returns:
%%	spacing - average spacing between receivers 
%%	freq_range - frequency range to use (min, max)
%%	chind - indices of the channels (traces) in 'rec' to use
%%	f - column of frequencies to use
%%	slwns - row of slownesses to use
%%	spec1 - matrix of spectra for the selected frequencies and channels

function [ spacing, freq_range, chind, f, slwns, spec1 ] = semblance_params(rec, spec)

global procdata

  % set offset range for fundamental mode 

  if isempty(procdata.offset_range)	% use all offsets
	offset_range = [ 0, max(rec.offset)*1.1 ];
  else
  	offset_range = procdata.offset_range(1,:);	
  end

  spacing = abs(rec.rec_x(end) - rec.rec_x(1))/abs(rec.chan(end) - rec.chan(1)); 
						% nominal spacing between receivers
      
  freq_range = [ 0, 0.5*procdata.phase_vel_expected/spacing ];

  slwns = linspace(1.0/procdata.phase_vel(2),1.0/procdata.phase_vel(1),200);	
						% linear grid of slownesses

%% form column of output frequencies (from 0 to Nyquist, within procdata.freq_range )

  f = transpose(0:(spec.fN-1))*spec.fi;

  skip_f = find(f < freq_range(1) | f > freq_range(2) ); % range of frequencies to skip

  % select records as: 	1) not killed, 
  %			2) within offset_range for fundamental mode
  %			3) within procdata.chan_range if specified
  %			4) within procdata.offset_range if specified
  % and truncate the spectra to the requested frequency range 

  aoffs = abs(rec.offset);
  maxoffs = max(aoffs);

  % mask for selected traces - skip traces marked as killed in 'rec' and in 'procdata'

  usechan = use_channels(rec.chan,rec.kill);

  sel = usechan & aoffs >= offset_range(1,1) & aoffs <= offset_range(1,2);

  if size(procdata.chan_range,1) >= 1

	sel1 = usechan & rec.chan >= procdata.chan_range(1,1) & rec.chan <= procdata.chan_range(1,2);

  	for i = 2:size(procdata.chan_range,1)	
		sel1 = sel1 | rec.chan >= procdata.chan_range(i,1) & rec.chan <= procdata.chan_range(i,2);
	end

	sel = sel & sel1;
  end

  if size(procdata.offset_range,1) >= 1

	sel1 = usechan & aoffs >= procdata.offset_range(1,1) & aoffs <= procdata.offset_range(1,2);

  	for i = 2:size(procdata.offset_range,1)	
		sel1 = sel1 | aoffs >= procdata.offset_range(i,1) & aoffs <= procdata.offset_range(i,2);
	end

	sel = sel & sel1;
  end

  chind = find(sel );			% indices of records to use
  dispsemb.chans = rec.chan(chind);	% output list of channels

  if isempty(chind)
	offset_range
	error("No channels selected for dispersion measurements")
	return
  end

  spec1 = spec.data(:,chind);

  spec1((spec.fN+1):end,:) = [];
  spec1(skip_f,:) = [];
  f(skip_f,:) = [];

end

