%% Calculates dispersion semblance structure 
%% =====================================================================
%%
%% Parameters:
%%	dsemb - structure containing dispersion spectra
%%	slice_no - if dsemb.semblance is 3-D, this parameter is the
%%		index of the 2-D slice being picked (default = 1)
%% Returns:
%%	p_alias - line showing aliasing range
%%	p_separ - line showing separation between modes and non-mode areas
%%	semb_peak - semblance peaks above mode separator. 
%%		This is a matrix with three columns containing:
%%			[ frequency, slowness, peak_semblance_value ].
%%	semb_peak1 - semblance peaks below mode separator
%%		This is a matrix with three columns containing:
%%			[ frequency, slowness, peak_semblance_value ].

function [ p_alias, p_separ, semb_peak, semb_peak1 ] = semblance_picks(dsemb,slice_no)

global procdata

  if nargin < 3
	slice_no = 1;
  end

  % calculate a curve to indicate aliasing in p

  p_alias = ones(length(dsemb.f),1)*max(dsemb.slwns);	% constant equal max slowness in plot
  p1 = 0.5*dsemb.f.^(-1)/dsemb.spacing;

  ind = find(p1 < max(dsemb.slwns));	% indices of the part within slowness plot range
  p_alias(ind) = p1(ind);

  % find semblance peaks lying below p_alias

  semb_peak = [];		% these will be peaks close to the fundamental mode 
  semb_peak1 = [];		% these will be other peaks

  ind1 = find(dsemb.f >= dsemb.freq_min & dsemb.f <= dsemb.mode_separ(1));	
						% picking frequencies below the separator
  ind2 = find(dsemb.f >= dsemb.freq_min & dsemb.f > dsemb.mode_separ(1));	
						% frequencies above the separator

  % locate peaks below separator frequency

  for i=1:length(ind1)

	i1 = ind1(i);

	ind = find(dsemb.slwns < p_alias(i1));  % slownesses below the aliasing curve

	if ~isempty(ind)

  		[ s, i2 ] = max(dsemb.semblance(i1,ind));

		semb_peak = [ semb_peak; dsemb.f(i1), dsemb.slwns(ind(i2)), s ];
	end
  end

  % locate peaks above separator frequency:   slowness(f) = s(2) - s(3)/(frequency - s(1))

  separ = dsemb.mode_separ(2) - dsemb.mode_separ(3)*(dsemb.f(ind2) - dsemb.mode_separ(1)).^(-1);

  p_separ = zeros(size(dsemb.f));		% array for plotting the separator
  p_separ(ind2) = separ;

  for i=1:length(ind2)

	i1 = ind2(i);

	ind = find(dsemb.slwns < p_alias(i1) & dsemb.slwns > separ(i) );	
			% slownesses below the aliasing curve and above separator

	if ~isempty(ind)				
  		[ s, i2 ] = max(dsemb.semblance(i1,ind,slice_no));

		semb_peak = [ semb_peak; dsemb.f(i1), dsemb.slwns(ind(i2)), s ];	
	end

	ind = find(dsemb.slwns < p_alias(i1) & dsemb.slwns <= separ(i) );	
			% slownesses below the aliasing curve and below separator
	if ~isempty(ind)
  		[ s, i2 ] = max(dsemb.semblance(i1,ind,slice_no));	

		semb_peak1 = [ semb_peak1; dsemb.f(i1), dsemb.slwns(ind(i2)), s ];
  						% these are counted as semb_peak1
	end
  end

%semb_peak
%semb_peak1

end

