%% Calculates frequency-dependent weights for a given midpoint and a record section
%%
%% Parameters:
%%	src_x - coordinate of the source for the records
%%	rec_x - array of receiver coordinates in the records
%%	f - column of frequencies

%% Returns:
%%	weight - weight matrix. Rows correspond to frequencies in 'f'
%%		and columns correspond to columns (elements) in array 'x'
%%	weight_cmp - matrix of weights with rows corresponding to frequencies and columns
%%		corresponding to the midpoints on procdata.cmp_loc.
%%		This matrix represents weights with which the spectra produced
%% 		from the given record contribute to the CMP grid

function [ weight, weight_cmp ] = semblance_weight(src_x, rec_x, f)

global procdata

  weight_cmp = zeros(length(f),length(procdata.cmp_loc));	% grid of output weight for all CMPs

  x1 = rec_x - src_x;		% signed offsets

  ax = abs(x1);			% unsigned offsets

  % find columns in 'weight_cmp' corresponding to 'weight'

  ind_cmp = zeros(length(rec_x),1);

  dx = reshape(procdata.cmp_loc,[],1);

  for i=1:length(rec_x)
	[~,ind_cmp(i)] = min(abs(dx-rec_x(i)));
  end

  % go through positive- and negative-offset branches, add weights of data points to weight_cmp

  for is = 0:1

    for i=1:size(weight_cmp,1)

	lambda2 = 0.5*procdata.offset_range_N*procdata.phase_vel_expected/(f(i,1)+1.0e-16);	
							% half wavelength at this frequency

    	% select distances to the right or left

    	if is			% positive

   	  mind = max([procdata.offset_range(1,1), min(ax(find(x1>0))) ]);

	  if lambda2 > 1.e-8
		maxd = min([procdata.offset_range(1,2), mind + lambda2 ]);
	  else
		maxd = procdata.offset_range(1,2);
	  end

    	else			% negative

    	  maxd = -max([procdata.offset_range(1,1), min(ax(find(x1<0))) ]);

	  if lambda2 > 1.e-8	
		mind = -min([procdata.offset_range(1,2), -maxd + lambda2 ]);
	  else
		mind = -procdata.offset_range(1,2);
	  end

    	end

	ind = find( x1 >= mind & x1 <= maxd );		% indices of channels to use

	% for each midpoint within the offset range, 
	% add a cosine-shaped weight function centered on the midpoint

	if lambda2 > 1.e-8

		b = pi/lambda2;	 % factor for Hann taper

		for j=1:length(ind)

			x0 = (rec_x(ind(j)) + src_x)*0.5;	% midpoint

			ind1 = find( procdata.cmp_loc > x0 - lambda2 ...
					& procdata.cmp_loc < x0 + lambda2 );	% interval of nonzero weight contribution

			weight_cmp(i,ind1) = weight_cmp(i,ind1) + (cos((procdata.cmp_loc(ind1)-x0)*b) + 1)*0.5;
		end

	else		% assign weight=1 to midpoints only

		weight_cmp(i,ind_cmp) = weight_cmp(i,ind_cmp) + 1;
	end
    end
  end

  % scale weights to sum = 1 at each frequency

  for j=1:size(weight_cmp,1)

	weight_cmp(j,:) = weight_cmp(j,:)/(sum(weight_cmp(j,:),2)+1.0e-16);
  end

  % extract output grid at midpoint positions

  weight = weight_cmp(:,ind_cmp);

%rec_x

  % make a test plot of weight functions 

  if procdata.plot_aux
	figure(777)
	clf
	hold on
	imagesc(procdata.cmp_loc,f,weight_cmp)
	plot( src_x, f(end), 'xo')
	colorbar
	set(gca,'Ydir','normal')
	xlabel('X, m')
	ylabel('Frequency, kHz')
	title(sprintf('Dispersion semblance weight for ffid = %d src_x = %1.1f m',procdata.run_ffid, src_x ))
	pause(1)
  end

end

