%%  Returns complex Fourier spectrum of one or multiple time series
%%	The time series are tapered by cosine tapers before FFT.
%%
%% Parameters:
%%      data - records for which spectra are returned. If data is a 2D matrix, each of its columns 
%%            is processed independently.
%%      dt - sampling interval in this record
%%	window - if this parameter is given, multitaper spectra are returned. Each taper has a length
%%		of 'window' (in true time units, presumably shorter than the total time interval), 
%%		with tapers overlapping by no less than 1/3.
%%		If multitaper spectra are output, their amplitude is averaged and phase is set equal zero
%%  Returns:
%%      spec - matrix of spectra of the same size as 'data'
%%      fi = frequency sampling interval
%%      fN - index of the Nyquist frequency in output

function [ spec, fi, fN ] = spectrum( data, dt, window )
   
  if isempty(data)
	spec = [];
	fi = [];
	fN = [];
	return
  end

  nd = size(data,1);    		% actual number of time points in data

  if nargin > 2						% doing multitaper
	nwin = ceil(window/dt);				% number of samples in one tapered window

	if nwin > nd			% no room for tapers
		nwin = nd;
		ntapers = 1;
	else
		ntapers = max(1, ceil((nd - nwin)/nwin*1.33333));	% number of tapers to do
	end

	wininc = floor((nd - nwin)/ntapers);		% window increment

  	taper = sin(linspace(0.0,pi,nwin))';	% time taper function

	spec = zeros(nwin,size(data,2));

	k = 1;

%nd
%window
%nwin
%wininc
%ntapers

	maxk = nd - nwin +1;

  	for i=1:ntapers

		if k > maxk		% make sure window does not move out of the record 
			k = maxk;
		end

		d1 = data(k:(k + nwin - 1),:).*taper;

  		spec = spec + abs(fft(d1)).^2;         % add FFT power of each column

		k = k + wininc; 

  	end

	spec = sqrt(spec);

  	fi = 1.0/(nwin*dt); 		% frequency sampling interval 

  else				% single spectrum 

  	taper = sin(linspace(0,pi,nd))';	% time taper function

  	spec = fft(data.*taper);         % FFT of each tapered column

  	fi = 1.0/(nd*dt); 		% frequency sampling interval 

  end
  
  if nargout > 2
    	fN = round(0.5*size(spec,1));  % Nyquist frequency index

%np = size(spec,1)
%fN

%figure(3)
%clf
%hold on
%plot(1:fN,real(spec(1:fN,:)),"b");
%plot(2:fN+1,real(spec(end:-1:fN+1,:)),"r")
%plot(1:fN,imag(spec(1:fN,:)),"g");
%plot(2:fN+1,-imag(spec(end:-1:fN+1,:)),"c")

  end
   
end


