% Sets global structure bfun containing  Wiggins' polynomial coefficients of various basis functions: 
% 	bfun.uz and bfun.uz for vertical- and radial-component displacement
% 	bfun.ezz, bfun.exz, bfun.exx for strain components
% 	bfun.szz, bfun.sxz, bfun.sxx for stress components
%
% 	Each function is presented by a 3D array with dimensions: 
%		(interval,row_p,number_of_basis_function);

%	Each 'row_p' here consists of layer number followed by four polynomial coefficients for basis function 
%
%	For each row_p, the basis function is given by expression:
%		fun = polyval(row_p(2:end),zeta),
%	where zeta is the depth coordinate normalized so that it equals 0 at the upper
%	boundary of the layer and 1 at the lower boundary.
%
%	With regard to the third index of the arrays, odd values of 'number_of_basis_function'
%	contain Wiggins' "value" functions (with unit displacement on the boundaries),
%	and rows with even-valued 'number_of_basis_function'
%	contain Wiggins' "gradient" functions (with unit stress on the boundaries)
%
% Parameters:
%	kx - horizontal wavenumber for which the basis functions are constructed

function basis_rayleigh(kx)

global model bfun

  bfun.kx = kx;		% store the wavenumber in basis function structure

  % allocate four basis functions on each boundary except bottom, 
  % plus two basis functions on the free surface 

  bfun.uz = zeros(2,5,model.num_modes_PSV);	
  bfun.ux = bfun.uz;

  %  two zero-stress free-surface functions with unit displacement along Z and X, respectively:

  [ bfun.uz(1,:,1), bfun.ux(1,:,1), bfun.uz(1,:,2), bfun.ux(1,:,2) ] = bfun_displacement(1,0.0);

  nf = 3;	% number of next output function

  %  fill displacement functions for each internal boundary

  for i = 1:(model.nlayers-1)

	zb = model.z(i);	% depth of the boundary

	% start stress functions
	% halves of basis functions above boundary i:

	[ bfun.uz(1,:,nf), bfun.ux(1,:,nf), bfun.uz(1,:,nf+1), bfun.ux(1,:,nf+1) ] = bfun_stress(i, 1.0);

	% halves of basis functions below boundary i:

	[ bfun.uz(2,:,nf), bfun.ux(2,:,nf), bfun.uz(2,:,nf+1), bfun.ux(2,:,nf+1) ] = bfun_stress(i+1, 0.0);

	nf = nf + 2;			

	% start displacement functions

	% halves of basis functions above boundary i:

	[ bfun.uz(1,:,nf), bfun.ux(1,:,nf), bfun.uz(1,:,nf+1), bfun.ux(1,:,nf+1) ] = bfun_displacement(i, 1.0);

	% halves of basis functions below boundary i:

	[ bfun.uz(2,:,nf), bfun.ux(2,:,nf), bfun.uz(2,:,nf+1), bfun.ux(2,:,nf+1) ] = bfun_displacement(i+1, 0.0);

	nf = nf + 2;			

  end	% creating basis functions

  % init strain functions (only layer numbers are important here)

  bfun.ezz = bfun.uz;		
  bfun.exz = bfun.uz;
  bfun.exx = bfun.uz;
  bfun.delta = bfun.uz;		% this will be volumetric strain

  % fill strain functions

  for i=1:size(bfun.exx,3)
  	for j=1:size(bfun.exx,1)

		layer = bfun.exx(j,1,i);

		if layer > 0
			[ ez, ex ] = strain( layer, bfun.kx, bfun.uz(j,2:end,i), bfun.ux(j,2:end,i) );

			bfun.ezz(j,2:end,i) = ez(1,:);
			bfun.exz(j,2:end,i) = ez(2,:)+ex(2,:);
			bfun.exx(j,2:end,i) = ex(3,:);

			bfun.delta(j,2:end,i) = ez(1,:) + ex(3,:);
		end
	end
  end
  
  % calculate stress functions

  bfun.szz = bfun.uz;		
  bfun.sxz = bfun.uz;
  bfun.sxx = bfun.uz;

  for i=1:size(bfun.sxx,3)
  	for j=1:size(bfun.sxx,1)

		layer = bfun.sxx(j,1,i);

		if layer > 0
			stress = stress_strain( layer, [ bfun.ezz(j,2:end,i); bfun.exz(j,2:end,i); bfun.exx(j,2:end,i) ] );

			bfun.szz(j,2:end,i) = stress(1,:);
			bfun.sxz(j,2:end,i) = stress(2,:);
			bfun.sxx(j,2:end,i) = stress(3,:);
		end
	end
  end

end

