% Returns values of displacement basis function
% The function has displacement of magnitude 1 in component Z for bfz_z, bfz_x and
% component X bfx_z, bfx_x,
% zero the other component of displacement, and zeros in both components of stress.
% 
% Parameters:
%	layer - number of the basis function
%	zeta - normalized depth value within range [ 0,1 ]
% Returns:
%	bfz_z, bfz_x - basis functions with unit displacements along Z
%		in the form of  rows [ layer_number, four polynomial coefficients ]
%		bfz_z is for z component, and bfz_x is for x component.
%	bfx_z, bfx_x - basis functions with unit displacements along X,
%		in the same format

function [ bfz_z, bfz_x, bfx_z, bfx_x ] = bfun_displacement(layer,zeta)

global model bfun

  w1 = [ -2, 3, 0, 0 ];		% Wiggins functions
  w2 = [ 1, -1, 0, 0 ];
  w3 = [ 2, -3, 0, 1 ];
  w4 = [ 1, -2, 1, 0 ];

  if zeta < 0.5		% boundary at the top of layer

  	[ a, b ] = stress( layer, bfun.kx, w3, w3, zeta );
  	[ c, d ] = stress( layer, bfun.kx, w4, w4, zeta );

	f = -inv([ c(1:2,1), d(1:2,1) ])*a(1:2,1);

	bfz_z = w3 + w4*f(1,1); 
	bfz_x = w4*f(2,1);

	f = -inv([ c(1:2,1), d(1:2,1) ])*b(1:2,1);

  	bfx_z = w4*f(1,1);
	bfx_x = w3 + w4*f(2,1); 

  else			% boundary at the bottom of the layer

  	[ a, b ] = stress( layer, bfun.kx, w1, w1, zeta );
  	[ c, d ] = stress( layer, bfun.kx, w2, w2, zeta );

	f = -inv([ c(1:2,1), d(1:2,1) ])*a(1:2,1);

	bfz_z = w1 + w2*f(1,1); 
	bfz_x = w2*f(2,1);

	f = -inv([ c(1:2,1), d(1:2,1) ])*b(1:2,1);

  	bfx_z = w2*f(1,1);
	bfx_x = w1 + w2*f(2,1); 
  end

  bfz_z = [ layer, bfz_z ];		% prepend layer number
  bfz_x = [ layer, bfz_x ];
  bfx_z = [ layer, bfx_z ];	
  bfx_x = [ layer, bfx_x ];

end

