% Returns values of stress basis function
% The function has stress of magnitude 1 in component 'comp',
% zero other stress component, and zero both components of displacement. 
% 
% Parameters:
%	layer - number of the basis function
%	zeta - normalized depth value within range [ 0,1 ]
% 
% Returns:
%	bfz_z, bfz_x - basis functions with unit displacements along Z
%		in the form of  rows [ layer_number, four polynomial coefficients ]
%		bfz_z is for z component, and bfz_x is for x component.
%	bfx_z, bfx_x - basis functions with unit displacements along X,
%		in the same format

function [ bfz_z, bfz_x, bfx_z, bfx_x ] = bfun_stress(layer,zeta)

global model bfun

  if zeta < 0.5		% boundary at the top of layer

	w4 = [ 1, -2, 1, 0 ];

  	[ c, d ] = stress( layer, bfun.kx, w4, w4, zeta );

	M = inv([ c(1:2,1), d(1:2,1) ]);

	f = M(:,1);		% this equals M*[1;0];

	bfz_z = w4*f(1,1); 
	bfz_x = w4*f(2,1);

	f = M(:,2);		% this equals M*[0;1];

  	bfx_z = w4*f(1,1);
	bfx_x = w4*f(2,1); 

  else			% boundary at the bottom of the layer

	w2 = [ 1, -1, 0, 0 ];
  	
	[ c, d ] = stress( layer, bfun.kx, w2, w2, zeta );

	M = inv([ c(1:2,1), d(1:2,1) ]);

	f = M(:,1);		% this equals M*[1;0];

	bfz_z = w2*f(1,1); 
	bfz_x = w2*f(2,1);

	f = M(:,2);		% this equals M*[0;1];

  	bfx_z = w2*f(1,1);
	bfx_x = w2*f(2,1); 
  end

  bfz_z = [ layer, bfz_z ];		% prepend layer number
  bfz_x = [ layer, bfz_x ];
  bfx_z = [ layer, bfx_z ];	
  bfx_x = [ layer, bfx_x ];

end

