% Returns the value(s) of one or multiple basis functions at one or multiple depths
%
% Parameters:
%	bf - the set of basis functions. This variable can be given in two ways:
%		1) By a character string specifying the type of basis function required:
%		'uz', 'ux' (for displacement components)
%		'ezz' (or 'ez'), 'exz' (or 'ezx'), 'exx' (or 'ex') (strain components)
%		'szz' (or 'sz'), 'sxz' (or 'szx'), 'sxx' (or 'sx') (stress components);
%		2) directly by one of the 3-D matrices bfun.uz, bfun.ux, bfun.ezz, bfun.exx, etc.
%	bfnum - number of the basis function. If this field equals [], 
%		all available basis functions are used.
%	z - 1-D or 2-D matrix of depths at which to make outputs.
%
% Returns:
%	v - values of the basis functions at level(s) in 'z'.
%		Each column of this matrix represents a function of the corresponding column in z.
%		The resulting size(v) is: 
%			[size(z,1), size(z,2), optional number of basis functions in bfnum ]

function v = bfun_value( bf, bfnum, z )

global model bfun

  % if bf is a mnemonic string, extract all basis functions of the required type
 
  if ischar(bf)
  	bf = bfun_extract( bf, bfnum );
  end

  bfnum = size(bf,3);

  v = zeros(size(z,1),size(z,2),bfnum);		% output column

  % go through all layers and fill the corresponding depth intervals in v

 for i=1:size(z,2)
   for k=1:bfnum
      for j = 1:size(bf,1)

  	layer = bf(j,1,k);	% layer number

	if layer < 1		% skip empty records
		continue
	end

	if layer > 1
		z0 = model.z(layer-1,1);
	else
		z0 = model.z0;
	end

	ind = find( z(:,i) >= z0 & z(:,i) <= model.z(layer,1) );

	if ~isempty(ind)
		v(ind,i,k) = polyval(bf(j,2:end,k),(z(ind,i) - z0)/(model.z(layer) - z0));
	end
      end		% for j
    end		% for k
  end		% for i
end

