% Function returning the energy forms for the medium 
% with basis functions corresponding to P/SV particle motions
% 
% in the globally-defined velocity/density model.
%
% Returns:
%	M - mass matrix, defined so that the kinetic energy equals
%		Ekinetic = (1/2)*(v'*M*v), 
%			where v is the vector of time derivatives of
%			magnitudes of basis functions (denoted u below);
%
%	S - stiffness matrix, defined so that the elastic energy equals
%		Eelastic = (1/2)*(u'*S*u),
%			where u is the vector of magnitudes of basis functions;
%
%	D - (optional) damping matrix, defined so that the dissipation function equals
%		Diss = (1/2)*(v'*D*v).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ M, S, D ] = energy_forms_PSV()

global model bfun

  nf = size(bfun.ux,3);	% number of basis functions

  M = zeros(nf,nf);	% Initialize zero for output matrices
  S = zeros(nf,nf);

  if nargout >= 3		% need dissipation
	D = zeros(nf,nf);
  end	
  	

%z
%kreal

  % Evaluate elements of energy matrices for all pairs of basis functions

  for i=1:nf

    % find the range of functions that may overlap

    jstart = i;

    jend = i+8;

    if jend > nf 
	jend = nf;
    end

    for j=jstart:jend

	% find matching layers

	[ ind, indj ] = ismember(bfun.ezz(:,1,i),bfun.ezz(:,1,j));

	indi = find(ind);

	if isempty(indi)	% no overlap
		continue;
	end

	indj = indj(indi);	% drop zero values

	for k=1:length(indi)

		li = indi(k);
		lj = indj(k);

		layer = bfun.uz(li,1,i);	% layer in which the basis functions overlap

		if layer < 1 		% skip empty records
			continue
		end

		u2 = integral(layer,conv(conj(bfun.uz(li,2:end,i)),bfun.uz(lj,2:end,j)) + ...
					conv(conj(bfun.ux(li,2:end,i)),bfun.ux(lj,2:end,j)));

		M(i,j) = M(i,j) + 0.5*model.rho(layer,1)*u2;

		delta2 = integral(layer,conv(conj(bfun.delta(li,2:end,i)),bfun.delta(lj,2:end,j)));

		eps2 = integral(layer,conv(conj(bfun.ezz(li,2:end,i)),bfun.ezz(lj,2:end,j)) + ...
					2*conv(conj(bfun.exz(li,2:end,i)),bfun.exz(lj,2:end,j)) + ...
					conv(conj(bfun.exx(li,2:end,i)),bfun.exx(lj,2:end,j)));

		S(i,j) = S(i,j) + 0.5*model.lambda(layer,1)*delta2 + model.mu(layer,1)*eps2;

		if nargout >= 3
			D(i,j) = D(i,j) + 0.5*model.eta_lambda(layer,1)*delta2 + model.eta_mu(layer,1)*eps2;
		end

	end	% for k

	if j > i		% fill the lower triangle by conjugates
		M(j,i) = conj(M(i,j));
		S(j,i) = conj(S(i,j));

		if nargout >= 3
			D(j,i) = conj(D(i,j));
		end
	end
    end 	% for j

  end   % for i

end	% function

