%% Loads model and models phase velocities for surface-wave modes
%% Outputs are placed in structure 'modes' saved in .mat file
%% ===========================================================================

%% Parameters:
%%	line_no - line number
%%	vintage - vintage number for the model
%%	ID - location of the model (1 or 15 in the present experiment)
%% 	lambda2 - optional parameter: 
%%		array of HALF-LENGTHS (approximately depths of investigation)
%%		for modeling. If this parameter is not set, current model is only loaded
%%		from .mat file
%%	scale_V - optional column of scaling factors for P- and S-wave velocities
%%		applied to the model
%%
%% Returns:
%%	matfile - name of .mat file in which the model and modes are saved

function matfile = forward_model( line_no, vintage, ID, lambda2, scale_V )

global procdata
global model bfun modes inversion

	recompute = false;	% flag to recompute kernels

	matfile = filename_model( line_no, vintage, ID);	% file name to load and save the model

	%% if file exists, load the model and inversion parameters from it

	if exist(matfile)
		load(matfile)
	else			% create initial model

		%% Set up new model 
		%%	Columns are depth at bottom, density, P-wave velocity, S-wave velocity, 
		%%		P-wave viscosity, S-wave viscosity (don't worry about the last two))

		if length(procdata.starting_model) >= 3	

			fn = filename_model( procdata.starting_model(1), procdata.starting_model(2),...
							procdata.starting_model(3))

			if exist(fn)	% load model from .mat file from (possibly) another line
				
				load(fn)

			else	% use default text file

				fn = procdata.default_model;

				if ~exist(fn)
					fn = 'interp/default_model.txt';

					msgbox(sprintf('File %s does not exist.\nUsing %s', procdata.default_model, fn));
				end

				model0 = load(fn,'-ascii');

				set_model(model0,ones(size(model0,1),1));	% initial model

				recompute = true;
			end

		else	% from text file

			fn = procdata.default_model;

			if ~exist(fn)
				fn = 'interp/default_model.txt';

				msgbox(sprintf('File %s does not exist.\nUsing %s', procdata.default_model, fn));
			end

			model0 = load(fn,'-ascii');

			set_model(model0,ones(size(model0,1),1));	% initial model

			recompute = true;
		end
	end

	% Adjust global variable 'model' using the velocity-density columns above:

	if nargin >= 5				% model scaling requested
		scale_model(scale_V,true);

		recompute = true;
	end

	% Model phase and group velocities at wavenumbers corresponding to lambda2:

	if recompute 
		if nargin >= 4
			modes = modes_rayleigh( pi*lambda2.^(-1) );
		else
			modes = [];
		end
	end


	modes.id = ID;
	modes.vintage = vintage;

%recompute
%modes

	%% save model and all previous data in .mat file 
  
	if inOctave()
  		save(matfile,'-mat7-binary','model','modes');
	else
  		save(matfile,'model','modes');
	end

end


