%% Setting up the data and starting model for inversion
%% -------------------------------------------------------------------------
%%
%% Results of this function are placed in structure 'inversion':
%%	inversion.data0 - matrix of fundamental-mode data:
%%		the first column is the frequencies,
%%		and the second column is the phase velocities to be are matched
%%	inversion.data1 - matrix of the data for the first overtone, in the same format.
%%		Usually this matrix would be empty.
%%	inversion.data2 - matrix of the data for the second overtone, in the same format.
%%		Usually this matrix would be empty.
%%	inversion.kx - array of wavenumbers at which the modeling and inversion should be
%%		performed.
%%
%% Parameters:
%%	data0 - fundamental-mode phase-velocity data in the form of a matrix with two
%%		columns: frequencies and velocity values
%%	data1 - optional data for first overtone (value [] means no such data)
%%	data2 - optional data for second overtone (value [] means no such data)
%%
%% ================================================================================

function inv_init(data0, data1, data2 )

global model inversion

  inversion.num_modes = 1;	% number of modes to invert; default is one

  % in inversion.data0 and similar matrices, add second column of wavenumbers

  if nargin < 1	  
	data0 = [];
  end

  if nargin < 2	  
	data1 = [];
  end

  if nargin < 3	  
	data2 = [];
  end

  % determine the frequency band for modeling using min and max frequencies in the data

  if isempty( data0 )
    	freq = [];
  else
	freq = [ min(data0(:,1)), max(data0(:,1)) ];
	v = [ min(data0(:,2)), max(data0(:,2)) ];
  end

  if ~isempty( data1 )
	freq = [ min([ data1(:,1); freq(1) ] ), max( [ data1(:,1); freq(2) ] ) ];
	v = [ min([ data1(:,2); v(1) ] ), max( [ data1(:,2); v(2) ] ) ];

	inversion.num_modes = 2;	% set number of modes to invert
  end

  if ~isempty( data2 )
	freq = [ min([ data2(:,1); freq(1) ] ), max( [ data2(:,1); freq(2) ] ) ];
	v = [ min([ data2(:,2); v(1) ] ), max( [ data2(:,2); v(2) ] ) ];

	inversion.num_modes = 3;	% number of modes to invert
  end

  if isempty(freq)	% no data, no need to invert

	inversion.data0 = [];
	inversion.data1 = [];
	inversion.data2 = [];

	return
  end

  % using the data, determine the set of wavelengths 
  % for modeling phase velocities

  lambda_min = 0.8*v(1)/freq(2);	% shortest wavelength, with some margin 
  lambda_max = 1.25*v(2)/freq(1);	% longest wavelength 

  % wavenumber grid for inversion-related operations

  %inversion.kx = linspace(2*pi/lambda_max,2*pi/lambda_min,40);		% old grid linear in kx

  % make the grid linear in sqrt(wavelengths):

  inversion.kx = 2*pi*linspace(sqrt(lambda_min),sqrt(lambda_max),40).^(-2);

	% this way, the sensitivity kernels are sampled more uniformly (see plot_gradients())

  % resample the data to match the grid of kx:

  inversion.data0 = inv_data(data0);
  inversion.data1 = inv_data(data1);
  inversion.data2 = inv_data(data2);

  % assign relative data weights for the two overtiones (equal 1.0 for now)

  inversion.data_weight = [ 1.0, 1.0 ];

end

