%% For a set of basis functions, returns the norm matrix 
%% that can be used for evaluating projections in model space.
%% The norm matrix is defined as:
%%
%%	M_ij = integral_dz( conj(bfun_i) * bfun_j )
%%
%% where bfun_i is the basis function
%% 
%% Parameters:
%%	bfz - the set of basis functions for Z component. This variable can be given in two ways:
%%		1) By a character string specifying the type of basis function required:
%%		'uz', 'ux' (for displacement components)
%%		'ezz' (or 'ez'), 'exz' (or 'ezx'), 'exx' (or 'ex') (strain components)
%%		'szz' (or 'sz'), 'sxz' (or 'szx'), 'sxx' (or 'sx') (stress components);
%%		2) directly by one of the 3-D matrices bfun.uz, bfun.ux, bfun.ezz, bfun.exx, etc.
%%	bfx - the set of basis functions for X component. This variable can be given in two ways:
%%		1) By a character string specifying the type of basis function required:
%%		'uz', 'ux' (for displacement components)
%%		'ezz' (or 'ez'), 'exz' (or 'ezx'), 'exx' (or 'ex') (strain components)
%%		'szz' (or 'sz'), 'sxz' (or 'szx'), 'sxx' (or 'sx') (stress components);
%%		2) directly by one of the 3-D matrices bfun.uz, bfun.ux, bfun.ezz, bfun.exx, etc.
%%	bfzx - (optional) the set of basis functions for XZ component. 
%%		This parameter is only needed when evaluating the norm for strain and stress tensors.
%%		This variable can be given in two ways:
%%		1) By a character string specifying the type of basis function required:
%%		'uz', 'ux' (for displacement components)
%%		'ezz' (or 'ez'), 'exz' (or 'ezx'), 'exx' (or 'ex') (strain components)
%%		'szz' (or 'sz'), 'sxz' (or 'szx'), 'sxx' (or 'sx') (stress components);
%%		2) directly by one of the 3-D matrices bfun.uz, bfun.ux, bfun.ezz, bfun.exx, etc.
%% Returns:
%%	M - matrix M above 

function M = inv_norm( bfz, bfx, bfzx )

global model bfun

  nf = size(bfun.uz,3);		% number of modes obtained from current bfun
				% so that this code should work for SH waves

  % if bf is a mnemonic string, extract all basis functions of the required type
 
  if ischar(bfz)
  	bfz = bfun_extract( bfz, 1:nf );
  end

  if ischar(bfx)
  	bfx = bfun_extract( bfx, 1:nf );
  end

  if nargin >= 3		% bfzx given
 	if ischar(bfzx)
  		bfzx = bfun_extract( bfzx, 1:nf );
  	end
  else
	bfzx = [];
  end

  M = zeros(nf,nf);

  % Evaluate elements of matrix M

  for i=1:nf

    % find the range of functions that may overlap

    jstart = i;

    jend = i+8;

    if jend > nf 
	jend = nf;
    end

    for j=jstart:jend

	% find matching layers

	[ ind, indj ] = ismember(bfz(:,1,i),bfz(:,1,j));

	indi = find(ind);

	if isempty(indi)	% no overlap
		continue;
	end

	indj = indj(indi);	% drop zero values

	for k=1:length(indi)

		li = indi(k);
		lj = indj(k);

		layer = bfz(li,1,i);	% layer in which the basis functions overlap

		if layer < 1 		% skip empty records
			continue
		end

		% polynomial form of integral of 

	  	p = polyint(conv(conj(bfz(li,2:end,i)),bfz(lj,2:end,j))) + ...
			polyint(conv(conj(bfx(li,2:end,i)),bfx(lj,2:end,j)));

		if ~isempty(bfzx)
	  		p = p + polyint(conv(conj(bfzx(li,2:end,i)),bfzx(lj,2:end,j)))*2;
		end

		M(i,j) = M(i,j) + polyval(p,1.0)-polyval(p,0.0);	% integral over layer thickness

	end	% for k

	if j > i		% fill the lower triangle by conjugates
		M(j,i) = conj(M(i,j));
	end
    end 	% for j

  end   % for i

end

