%% Evaluates predicted data sensitivities (Jacobian) and second derivatives 
%% (diagonal elements of the Hessian)
%% -------------------------------------------------------------------------
%% Prior to calling this function, inv_init(...) should be called to provide
%% the data and kx grid in structure 'inversion'.
%%
%% For each model layer, its velocities are scaled up and down and all surface-wave modes 
%% are modeled, which produces three measurements of phase velocities. For each mode and 
%% frequency, a parabola is fitted to these three points, which gives the gradient of 
%% the phase velocity and the second derivative. These derivatives are interpolated at 
%% the same frequencies.
%% 
%% The results of this function are placed in structure inversion:
%%	inversion.freq - matrix of frequencies at which each mode is modeled 
%%			(as output by modes_fun() below)
%%	inversion.v - matrix of velocities (usually phase velocities) for each mode modeled 
%%			at frequencies in inversion.freq
%%	inversion.v_d - 3-D matrix of derivatives:
%%			row_index = layer_number
%%			column_index = number of kx (range 1:length(inversion.kx))
%%			third_index = mode_number (range 1:nmodes), 
%%	inversion.v_d2 - matrix of second derivatives of inversion.v (same format)
%%
%% Parameters:
%%	modes_fun - name of the function modeling wavemodes,
%%		for example @modes_rayleigh. This function should take a vector
%%		of kx and return columns of frequencies and phase 
%% ================================================================================

function inv_perturb(modes_fun)

global model inversion

  if nargin < 1		% no options given
	opt_in = [];
  end

  parval = [0.9, 1.0, 1.11 ];	% parameter values at which each layer is tested

  % model the unperturbed case

  scale_model();

  [ inversion.freq, inversion.v ] = modes_fun(inversion.kx);	 

  nkx = size(inversion.v,2);	% number of kx (columns in the above outputs)

  nmodes = min( [ 3, size(inversion.v,1) ] );	
		% number of modes for which to calculate first and second derivatives
		% three modes is the largest value expected for inversion.num_modes

  % init work matrix for three versions of the predicted data

  vph = zeros(nmodes, nkx, 3);
  vph(:,:,2) = inversion.v(1:nmodes,:);	% unperturbed solution

  % init matrices to store gradients and second derivatives

  inversion.v_d = zeros(model.nlayers, nkx, nmodes );			
  inversion.v_d2 = inversion.v_d;

  % perturb the model by +-10% twice for each model layer

  for i=1:model.nlayers

	par = ones(model.nlayers,1);

	for j=[-1,1]
	
		j2 = j+2;

		par(i,:) = parval(j2);		% scale i-th layer

		scale_model(par);

		% model phase velocities 

		[ ~, vphase ] = modes_fun(inversion.kx);

		vph(:,:,j2) = vphase(1:nmodes,:);
	end

	% fit parabolas to vph for each kx and mode

	for k=1:nmodes
	  for j=1:nkx
		done = isfinite(vph(k,j,:));	% valid points

		% if all three points calculated, fit a parabola

		if sum(done) == 3

  			p = polyfit(parval,reshape(vph(k,j,:),1,[]),2);

			inversion.v_d2(i,j,k) = 2*p(1);
			inversion.v_d(i,j,k) = polyval(polyder(p),1.0);	% gradient at par = 1
		end
	  end
	end
  end

end

