%% Inversion of phase-velocity data using 'fminsearch' (simplex) algorithm
%% -------------------------------------------------------------------------
%% Prior to calling this function, inv_init(...) should be called to provide
%% the data and wavelength grid in structure 'inversion'.
%%
%% Starting from the current model in global structure 'model', VS and VP 
%% velocities in each layer are scaled proportionally (VP -> VP*factor, 
%% VS -> VS*factor, with the same 'factor'), basis functions and phase velocities 
%% are modeled, and L1 (absolute-value) misfit from the data is measured.
%%
%% The misfit is then minimized by the simplex method in order to minimize 
%% phase-velocity misfits for all modes for which the inversion.data matrices are
%% not empty.
%%
%% The results of inversion are placed in the same 'model', and quality-control 
%%	 values are placed in structure 'inversion':
%%		 inversion.fminsearch.fval
%%
%% Parameters:
%%	misfit_function - name of data misfit function to use, 
%%				for example @vphase_misfit_rayleigh
%%	opt_in - optional structure of algorithm parameters for function 'fminsearch'
%%			opt_in = [] means no additional options passed to fminsearch
%%
%% Return values:

%% ================================================================================

function inv_simplex(misfit_function,opt_in)

global model inversion

  if nargin < 2		% no options given
	opt_in = [];
  end

  % perform simplex inversion

  inversion.Phi_best = Inf;			% initial value of the objective function 
						% in the best-fit model

  inversion.par = ones(model.nlayers,1);	% starting model (par=1 means model unscaled)

  if isempty(opt_in)
    [inversion.par,inversion.fminsearch.fval ] = fminsearch(misfit_function,inversion.par);
 
  else
    [inversion.par,inversion.fminsearch.fval ] = fminsearch(misfit_function,inversion.par,opt_in);
  end

  % scale the model by the final solution

  scale_model(inversion.par);

end

