%% Modeling Rayleigh-wave velocities in a layered structure described
%% in global structure 'model' 

%% Parameters:
%%	kx - array of horizontal wavenumbers for modeling
%% Return values:
%%	modes - structure with fields:
%%		modes.freq - matrix of complex-valued frequencies for each mode, 
%%			Columns of this matrix correspond to the wavenumbers (elements of kx). 
%%			Rows of each column correspond to increasing wave mode orders. 
%%		modes.vphase - matrix of complex-valued phase velocities, formatted as 'modes.freq' 
%			If wavenumber bfun.kx==0, values vphase = 0 are output
%%		modes.vgroup - matrix of complex-valued group velocities, formatted as 'modes.freq'
%%			If wavenumber bfun.kx==0, values vgroup = 0 are output

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function modes = modes_rayleigh(kx)

global model bfun procdata

  % create progress indicator

  if procdata.progress_indicator
	cf = gcf();			% current figure
  	wh = waitbar(0.0,'Modeling Raylegh wave modes','Name','Processing');
  else
	wh = [];
  end

  nk = length(kx);

  % initialize output matrices

  modes.freq = zeros(model.num_modes_PSV,nk);	% output frequencies, row index is mode number
  modes.vphase = modes.freq;			% phase velocities

%dlam = model.lambda - model.lambda0
%dmu = model.mu - model.mu0

  % for each wavenumber, output frequencies and phase velocities for each mode
  % basis functions are set for each wavenumber in global variable 'bfun'

  for i=1:nk
  	basis_rayleigh(kx(i));
	[ modes.vphase(:,i), modes.freq(:,i) ] = vel_rayleigh();

	if ~isempty(wh)
		waitbar(i/nk,wh);		% progress indicator
	end
  end

  if nargout >= 3		% need group velocities

    modes.vgroup = zeros(size(modes.freq));

    for i=1:size(modes.freq,1)
    	ind = find(modes.vphase(i,:) > 1.0e-8);  % columns in which kx != 0

      	modes.vgroup(i,ind) = vel_group(modes.freq(i,ind),modes.vphase(i,ind));
    end
  end

  if ~isempty(wh)
 	delete(wh);
	figure(cf);	% return to current figure
  end

end

