%% Interactive picking of velocity - density model fitting observed
%% phase velocities
%% ==============================================================================
%% For a given line number, ID (location), and vintage, the program displays data 
%% the available data as plot_modes_line(), models and diaplays dispersion curves, 
%% and allows adjusting velocities so that the true velocity profiles approximate 
%% the pseudodepth profiles for the fundamental mode (which, in turn, represent the   
%% observed phase velocity curves/
%%
%% Figure number (1000 + line_number) is used for picking, in order to avoid 
%% overlap with other tools.
%%
%% Parameters:
%%	line_no - line number
%%	vintage - vintage number for the model
%%	ID - location of the model (1 or 15 in the present experiment)
%% 	lambda2 - optional parameter: 
%%		array of HALF-LENGTHS (approximate depths of investigation)
%%		for modeling. If this parameter is not set, current model is only loaded
%%		from .mat file
%% =====================================================================

function pick_model(line_no, vintage, ID, lambda2 )

global procdata model modes sett

  % obtain settings in structure 'sett'
  
  if exist('../config/settings.m')
  	run('../config/settings.m')

  else		% create simple settings

	sett.window_wide = [100 100 1800 800];   % position (X,Y) in pixels and size (X,Y) of window 
  end

  matfile = forward_model( line_no, vintage, ID, lambda2 );

  nmodes = min([size(modes.freq,1), procdata.plot_modes_num]);
	
  ref_mode = 1;

  recompute = false;		% flag to recompute modes

  % lists for menus

  menu_list = {};

  for i=1:nmodes
	menu_list{length(menu_list)+1} = num2str(i);
  end

  parameter_menu_list = { "Density", "Vp", "Vs", "Vp/Vs ratio", "Layer depths", "All parameters" };

  fig = figure(1000+line_no);		% open a figure for this line
  clf
  set(gcf,'Position',sett.window_wide)

  title_frmt = 'Model Line %d, ID %d, Vintage %d\nRef. mode %d (d - delete; m - select mode; r - reload; s - save; q - quit; x - change parameters; z - undo edit; M1 - set Vs, M3 - scale Vs)';

  plot_axes(sprintf(title_frmt,line_no, ID, vintage, ref_mode))

  curr_version = 0;

  model_pred1 = [];	% previous models used for 'undoing' latest edits
  model_pred2 = [];
  model_pred3 = [];

  h = {};		% graphics handles for lines

  while 1

	% if not first run, re-compute phase velocities and plot the model

	if recompute

		modes = modes_rayleigh( pi*lambda2.^(-1) );
		recompute = false;
	end

	% clear curent plot

	for i=1:length(h)
		delete(h{i})
	end

	h = {};

	if strcmpi(procdata.plot_modes,'zvs')
 		h = plot_model1('ls');		% layered structure
	end

 	h1 = plot_model_modes1();		% modeled modes

	h = horzcat(h,h1);		% merge the lists of model lines

	plot_modes_line1(line_no, ID, vintage, 'k' );	% picks

	% set custom ticks if requested

  	if ~isempty(procdata.plot_Xtick_inc)		
		limits = xlim()/procdata.plot_Xtick_inc;

	  	set(gca,'Xtick', (ceil(limits(1)):floor(limits(2)))*procdata.plot_Xtick_inc )
  	end

  	if ~isempty(procdata.plot_Ytick_inc)		
		limits = ylim()/procdata.plot_Ytick_inc;

	  	set(gca,'Ytick', (ceil(limits(1)):floor(limits(2)))*procdata.plot_Ytick_inc )
  	end

	drawnow

	[v,z, buttonNumber] = ginput(1);
  
	% transform coordinates to (depth,velocity)

	switch lower(procdata.plot_modes)

		case 'fs'	% picked v is actually f, z is 1/v_phase
				
			f = v;
			v = 1.0/z;
			v = procdata.Vs*v;
			z = procdata.pseudodepth*v/f;

		case 'fv'	% picked v is actually f, z is v_phase
				
			f = v;
			v = z;
			v = procdata.Vs*v;
			z = procdata.pseudodepth*v/f;
	end

	switch buttonNumber

	  case 1		% left mouse button - set Vs in the model at the picked level

		% save backup copy of the model and modes

		if ~isempty(model_pred2)
			model_pred3 = model_pred2;
			modes_pred3 = modes_pred2;
		end

		if ~isempty(model_pred1)
			model_pred2 = model_pred1;
			modes_pred2 = modes_pred1;
		end

		model_pred1 = model;
		modes_pred1 = modes;

		% scale velocities

		layer = scale_model_Vs(z,v);

		% mark the scaled layer (to see it while doing modeling)

		vs = model.vs(layer,1);

		if layer > 1
			z = model.z(layer-1,1);
		else
			z = model.z0;
		end
 
		h{length(h)+1} = line([ vs, vs ], [ z, model.z(layer,1) ],'LineStyle','-','Marker','x','Color','g');

		drawnow

		curr_version = 0;	% mark model as edited
		recompute = true;	% ask to recompute surface-wave modes

	  case 3		% right mouse button - scale the model at the picked level

		% save backup copy of the model and modes

		if ~isempty(model_pred2)
			model_pred3 = model_pred2;
			modes_pred3 = modes_pred2;
		end

		if ~isempty(model_pred1)
			model_pred2 = model_pred1;
			modes_pred2 = modes_pred1;
		end

		model_pred = model;
		modes_pred = modes;
		
		% scale velocities

		layer = scale_model_fVs(z,v,ref_mode);

		% mark the scaled layer (to see it while doing modeling)

		vs = model.vs(layer,1);

		if layer > 1
			z = model.z(layer-1,1);
		else
			z = model.z0;
		end
 
		h{length(h)+1} = line([ vs, vs ], [ z, model.z(layer,1) ],'LineStyle','-','Marker','x','Color','g');

		drawnow

		curr_version = 0;	% mark model as edited
		recompute = true;	% ask to recompute surface-wave modes

	  case 'd'

		if curr_version
			query = questdlg(sprintf('Delete model version %d?', curr_version), ...
				sprintf('Line %d, ID %d, Vintage %d',line_no, ID, vintage), 'YES', 'NO', 'NO');

	       		if strcmpi(query,'yes')
		    		delete(filename_model( line_no, vintage, ID, curr_version));
			end
		else
			msgbox("Cannot delete the dafault or edited model.\nReload or save it as a numbered version first.");
		end

	  case 'm'		% select mode and start/stop picking 

		n = listdlg('PromptString','Select reference mode',...
				'SelectionMode','single','InitialValue',ref_mode,'ListString',menu_list);

		if ~isempty(n)
			ref_mode = n;

			title(sprintf(title_frmt,line_no, ID, vintage, ref_mode))	% refresh plot title
		end

	  case 'q'

	       query = questdlg('Exit without saving?', ...
			sprintf('Line %d, ID %d, Vintage %d',line_no, ID, vintage), 'YES', 'NO', 'NO');

	       if strcmpi(query,'yes')
		    break
		end

	  case 'r'		% restore model from file

		% obtain a list of available models

		[l,f] = list_model(line_no, vintage, ID);

		% form menu

		if length(l) > 1	% have some versions

			lst = {};

  			for i=2:length(l)

				lst{length(lst)+1} = sprintf('Version %d',l(i));
  			end

			n = listdlg('PromptString','Reload model','SelectionMode','single','InitialValue',1,'ListString',lst);

			if ~isempty(n)		%% user asked to load

				fn = f{n+1};

				if ~exist(fn)	% file does not exist (this should not really happen)

					msgbox(sprintf('File %s does not exist.\nCannot reload.', fn));

					break
				end

				curr_version = n;

				copyfile(fn,matfile);		% copy model into current one
				load(matfile);			% reload the model and modes

				str = sprintf('Reloaded version %d of %s', n, title_frmt);

				title(sprintf(str,line_no, ID, vintage, ref_mode))	% refresh plot title
			end
		else
			msgbox("No previous versions to reload");
		end

	  case 's'		% save model in file

		% obtain a list of available models and add a new one

		[l,f] = list_model(line_no, vintage, ID);

		newmod = max(l)+1;
		l = [ l, newmod ];
		ll = length(l);

		f{length(f)+1} = filename_model( line_no, vintage, ID, newmod);

		% form menu

		lst = {};

  		for i=1:ll

			if l(i) == 0
				lst{length(lst)+1} = 'Current model';
			elseif i == ll
				lst{length(lst)+1} = sprintf('Version %d (new)',l(i));
			else
				lst{length(lst)+1} = sprintf('Version %d',l(i));
			end
  		end

		n = listdlg('PromptString','Save model','SelectionMode','single','InitialValue',1,'ListString',lst);

		if ~isempty(n)		%% user asked to save

			if inOctave()
  				save(matfile,'-mat7-binary','model','modes');
			else
  				save(matfile,'model','modes');
			end

			if n > 1				%% save an archive copy	

				curr_version = n-1;
				copyfile(matfile,f{n});

				str = sprintf('Saved version %d of %s', curr_version, title_frmt);

				title(sprintf(str,line_no, ID, vintage, ref_mode))	% refresh plot title
			end
	       	end 

	  case 'x'	% update model from text file

		n = listdlg('PromptString','Modify parameters',...
				'SelectionMode','single','InitialValue',1,'ListString',parameter_menu_list);

		if ~isempty(n)

			[ fn, pth ] = uigetfile({'*.txt','Text files (*.txt)'; ...
							'*.mat;*.mod', 'Model files (*.mat,*.mod)';...
							'*.*', 'All files'},...
						 sprintf('Select file with %s data',parameter_menu_list{n}));

			if fn		% 'Cancel' selected in file dialog returns numeric 0
 
				layers = model_from_file( [ pth, fn ] );

				switch n	% { "Density", "Vp", "Vs", "Vp/Vs ratio", "Layer depth", "All parameters" };

					case { 1, 2, 3 }	% density, Vp, Vs

						modify_model(layers,n+1);
		
					case 4		% modify Vp by multiplying current Vs by Vp/Vs ratios from 'layers'

						layers(:,3) = interp1(model.z,model.vs,layers(:,1),'linear','extrap').*...
								layers(:,3)./layers(:,4);
							
						modify_model(layers,3);
			
					case 5		% layer depths

						modify_model(layers,1);

					case 6		% all parameters

						set_model(layers);
				end

				str = sprintf('Modified %s in %s', parameter_menu_list{n}, title_frmt);

				title(sprintf(str,line_no, ID, vintage, ref_mode))	% refresh plot title

				recompute = true;	% flag to recompute kernels
			end
		end

	  case 'z'	% undo edits

		% restore model and modes from backup

		if ~isempty(model_pred1)

			model = model_pred1;
			modes = modes_pred1;

			% pull the stack of backup models

			if ~isempty(model_pred2)

				model_pred1 = model_pred2;
				modes_pred1 = modes_pred2;

				model_pred2 = [];
			else
				model_pred1 = [];
			end

			if ~isempty(model_pred3)	

				model_pred2 = model_pred3;
				modes_pred2 = modes_pred3;

				model_pred3 = [];
			else
				model_pred2 = [];
			end
		end

		for i=1:length(h)
			delete(h{i});
		end

		h = {};

	  otherwise		% other keys

	end		% switch buttonNumber

  end		% big endless while loop

  close(fig);	% close figure

end

