%%%%%%%%%% P-SV wave modeling and inversion in a layered medium
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

% You can keep several (optional) versions of the model .
% Edit, for example, 'model_2' and keep other models for comparison

% Columns of these matrices are:
%	1) depth to the bottom of layer (m)
%	2) density
%	3) P-wave velocity (m/ms or km/s)
%	4) S-wave velocity
%	5) bulk viscosity
%	6) shear viscosity

model_1 = [
	0.5	1.53	0.188	0.094	0.0	0.0		
	2.0	1.53	0.2	0.1	0.0	0.0			
	5.0	1.13	0.23	0.115	0.0	0.0	
	8.0	1.13	0.238	0.119	0.0	0.0	
	15.0	1.13	0.3	0.15	0.0	0.0	
	35.0	1.53	0.3	0.15	0.0	0.0
];

model_2 = [
	0.5,	1.53,	0.192,	0.096,	0.0,	0.0		
	2.0,	1.53,	0.222,	0.111,	0.0,	0.0			
	5.0,	1.13,	0.232,	0.116,	0.0,	0.0	
	8.0,	1.13,	0.242,	0.121,	0.0,	0.0	
	15.0,	1.13,	0.3,	0.15,	0.0,	0.0	
	35.0,	1.53,	0.3,	0.15,	0.0,	0.0
];

% several global variables for data exchange between functions and interactive inspection
 
global model bfun inversion freq vphase vgroup

% Set up global 'model' using one of the velocity-density columns above:

set_model(model_2);

% construct all basis functions for some given wavenumber

basis_rayleigh(2*pi/1.0);	% for wavelength = 1 m

% The following calls can be used for preliminary testing. 
% They are not required for subsequent modeling:

plot_bfun_PSV();	% plot basis functions for wavelength 1 m 

plot_MSD_PSV();		% plot M, S, and D matrices

% plot a section of uz for a wave traveling at 1 m/ms downward

z = linspace(model.z0,10,20);	% 20 points to the depth of 20 m,
				% where layering ends in the current model			

c = bfun_spike('uz',z );	
		% for each level 'z(i)', this creates a model vector in 'c(:,i)' 
		% containing a decomposition of a unit spike at depth z(i)
		% in terms of the basis functions bfun.uz

plot_c(c);	% test plot showing scaled values of c for different wave mode numbers

figure(2);
	
plot_field('uz',c)	% plot uz wavefield of the decomposed spikes 

% Begin Rayleigh-wave modeling demo
% =====================================================================
% set up an array of desired HALF-WAVELENGTHS for output (in meters)
% These half-wavelengths are roughly the expected depths of sampling

lambda2 = [ 0.5 : 0.25 : 20 ];	

% Model phase and group velocities at wavenumbers corresponding to lambda2:

[ freq, vphase, vgroup ] = modes_rayleigh( pi*lambda2.^(-1) );

%% save all results in a .mat file 

save 'result_rayleigh.mat' model* freq v*

%% plot in figure 101 phase velocities and slownesses for six lowest modes

plot_model_modes( freq, vphase, [], 101 )

%% you can also plot group velocities, but they look complicated sometimes 

% plot_model_modes( freq, vgroup )

% Begin inversion demo (not completed yet!)
% =====================================================================
%% test of inversion using the phase velocities modeled above as data

inv_init([ transpose(freq(1,:)), transpose(vphase(1,:)) ]);

inv_simplex(@vphase_misfit_rayleigh);

%% perturb the model for each layer and 
%% calculate gradients of Rayleigh-wave phase velocities with respect to 
%% model scaling parameters

inv_perturb(@modes_rayleigh);

%% save inversion results in a .mat file 

save 'inversion_rayleigh.mat' model inversion

%% plot in figure 201 phaase-velocity gradients with respect to model scaling

plot_gradients(201)

%% perform iterative inverse by Newton iteration

inv_step(@modes_rayleigh, @inv_newton)

