%% Set global variable model based on the layer matrix
%%
%% Parameters:
%%	layers - layer parameter matrix with columns:
%%			1) depth to the bottom of layer
%%			2) density
%%			3) P-wave velocity
%%			4) S-wave velocity
%%			5) P-wave viscosity
%%			6) S-wave viscosity
%%	scale_V - optional column with the same number of rows as 'layers'
%%		If this parameter is given and is not empty, 
%%		Vp and Vs velocities in each layer are
%%		multiplied by rows of this column
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function set_model(layers, scale_V)

global model

  if nargin > 1
	if ~isempty(scale_V)

		if size(scale_V,1) ~= size(layers,1)
			error('Column scale_V should contain %d elements (as in model table)',size(layers,1))
		end

		layers(:,3:4) = layers(:,3:4) .* scale_V;	% scale the velocities
	end
  end

  model.nlayers = size(layers,1);

  % number of Rayleigh-wave modes:

  model.num_modes_PSV = 2+(model.nlayers-1)*4;	

  model.z0 = 0.0;	% top of model always at zero for now

  model.z = layers(:,1);	% depth to the bottom of each layer

  model.dz = [ layers(1,1)-model.z0; layers(2:end,1)-layers(1:(end-1),1) ]; % depth increments 

  model.rho = layers(:,2);	% density
  model.vp = layers(:,3);	% P-wave velocity
  model.vs = layers(:,4);	% S-wave velocity

  model.mu = model.vs.^2 .* model.rho;
  model.lambda = model.vp.^2 .* model.rho - 2.0 * model.mu;
  model.l2mu = model.lambda + 2.0*model.mu;

  model.eta_mu = layers(:,6);					% shear viscosity
  model.eta_lambda = layers(:,5) - 2.0/3.0 * model.eta_mu;	% lambda-type viscosity

  set_model0();		% finalize

end

